--local StatLogic = LibStub:GetLibrary("LibStatLogic-1.1")
local StatLogic = LibStub("LibStatLogic-1.2")
local Toolkit = LibStub:GetLibrary("Toolkit-1.0");

HealPointsCalculator ={
	HP_LastPowerfulCurrent = nil;
	--HP_LastPowerfulNew = nil;
	HP_LastEfficientCurrent = nil;
--	HP_LastEfficientNew = nil;
};

--[[
	function GetHPTick	
--]]

local BonusPercentInt;
local BonusPercentSpi;

-- Helper functions
--[[
	Returns the global cooldown time (normally 1.5 seconds) after
	it has been reduced by specifid amount of spell haste
--]]
local function getGCD(spellHaste)
	if (spellHaste == nil) then
		return 1.5;
	else
		return math.max(1.0, 1.5 / (1 + spellHaste / 100));
	end
end

--[[
	Get the amount of healing done by a particular spell
	@param spell a HealPointsSpells.SPELLTABLE entry the indicates the spell to be calculated
	@param A bonus amount of healing to be added to the spell
	@param number The healing crit chance
--]]
local function getHPCast(spell, healing, spellCrit)

	assert(spell, "calculator.lua:getHPCast - spell is nil");
	assert(healing, "calculator.lua:getHPCast - healing is nil");
	assert(spellCrit, "calculator.lua:getHPCast - spellCrit is nil");

	local percentBonus = HealPointsSpells:getHealingPercentBonus(spell);
	local healFactor, healFactor2 = HealPointsSpells:getHealingFactor(spell);
	spellCrit = spellCrit + HealPointsSpells:getSpellCritBonus(spell);
	healing = healing + HealPointsSpells:getHealingBonus(spell);

	local out, minHeal, maxHeal;

	if (spell.class['type'] == 'normal' or spell.class['type'] == 'instant') then
		minHeal = (spell['min'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		maxHeal = (spell['max'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);

		--out = (((spell['min'] + spell['max']) / 2) + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		out = (minHeal+maxHeal) / 2;

	elseif (spell.class['type'] == 'hot') then
		local healTick = spell['tick'] + healing * healFactor / spell.class['tickcount']; -- +healing divided between original tickcount
		out = healTick * HealPointsSpells:getNumberOfTicks(spell) * percentBonus;

		minHeal = out;
		maxHeal = out;

	elseif (spell.class['type'] == 'regrowth') then

		-- Burst part
		minHeal = (spell['min'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		maxHeal = (spell['max'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);

		--out = (((spell['min'] + spell['max']) / 2) + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		out = (minHeal+maxHeal) / 2;

		-- HOT part
		local healTick = spell['tick'] + healing * healFactor2 / spell.class['tickcount'];
		out     = out     + healTick * HealPointsSpells:getNumberOfTicks(spell) * percentBonus;
		minHeal = minHeal + healTick * HealPointsSpells:getNumberOfTicks(spell) * percentBonus;
		maxHeal = minHeal + healTick * HealPointsSpells:getNumberOfTicks(spell) * percentBonus;

	elseif (spell.class['type'] == 'lifebloom') then
		-- HOT part
		local healTick = 
				spell['tick']
				--+ (healing + 2.777 * HealPointsBS:GetBonus('AVG_HOT_LIFEBL')) * healFactor / spell.class['tickcount'];
		out = healTick * HealPointsSpells:getNumberOfTicks(spell) * percentBonus;
		-- Burst part
		out = out + (
				spell['burst']
				--+ (healing + HealPointsBS:GetBonus('AVG_BURST_LIFEBL')) * healFactor2
				) * percentBonus * (1 + spellCrit / 200);

	elseif (spell.class['type'] == 'chain') then
		minHeal = (spell['min'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		maxHeal = (spell['max'] + healing * healFactor) * percentBonus * (1 + spellCrit / 200);
		--out = (((spell['min'] + spell['max']) / 2) + healing * healFactor) * percentBonus * (1 + spellCrit / 200);

		minHeal = minHeal * 1.75; -- Assumes that the spell hits all 3 targets
		maxHeal = maxHeal * 1.75; -- Assumes that the spell hits all 3 targets

		--out = out * 1.75; -- Assumes that the spell hits all 3 targets
		out = (minHeal + maxHeal) / 2;

	elseif (spell.class['type'] == 'aoe') then
		out = (((spell['min'] + spell['max']) / 2) + healing * healFactor) * percentBonus * (1 + spellCrit / 200)
		out = out * math.min(5, HealPoints.db.char.hot['numtargets']);
	end

	if (_G["HealPointsOptionsConfigDivinePlea"]:GetChecked() == 1) then -- Divine plea
		-- 50% reduction 15/60 of the time = 12.5% reduction on average
		out = out * 0.875;
	end

  	return out, minHeal, maxHeal;
end

--[[
	Get HP healed per tick of a HOT spell
	--]]
function HealPointsCalculator:GetHPTick(spell, healing)
	local percentBonus = HealPointsSpells:getHealingPercentBonus(spell);
	local healFactor, healFactor2 = HealPointsSpells:getHealingFactor(spell);
	healing = healing + HealPointsSpells:getHealingBonus(spell);

	local healTick = 0;

	if (spell.class['type'] == 'hot') then
		healTick = spell['tick'] + healing * healFactor / spell.class['tickcount']; -- +healing divided between original tickcount

	elseif (spell.class['type'] == 'regrowth') then
		healTick = spell['tick'] + healing * healFactor2 / spell.class['tickcount'];

	elseif (spell.class['type'] == 'lifebloom') then
		healTick = 
				spell['tick']
				--+ (healing + 2.777 * HealPointsBS:GetBonus('AVG_HOT_LIFEBL')) * healFactor / spell.class['tickcount'];
	end

	healTick = healTick * percentBonus;
	return healTick;
end

--[[
	Calculate the amount of healing per second done by a spell
--]]
local function getHPSec(spell, healing, spellCrit, spellHaste)
	if (spell.class['type'] == 'normal' or spell.class['type'] == 'chain' or spell.class['type'] == 'aoe') then
		local castTime = HealPointsSpells:GetCastTime(spell, spellHaste);
		if (castTime == 0) then
			castTime = getGCD(spellHaste);
		end
		if (spell.class['cooldown'] ~= nil) then
			castTime = math.max(castTime, spell.class['cooldown']);
		end
		return getHPCast(spell, healing, spellCrit) / castTime;
	elseif (spell.class['type'] == 'instant') then
		return getHPCast(spell, healing, spellCrit) / spell.class['cooldown'];
	elseif (spell.class['type'] == 'hot' or spell.class['type'] == 'lifebloom') then
		return getHPCast(spell, healing, spellCrit) / (HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval']);
	elseif (spell.class['type'] == 'regrowth') then
		return getHPCast(spell, healing, spellCrit) / (HealPointsSpells:GetCastTime(spell, spellHaste) + HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval']);
	end
end

--[[
	Calcuate the amount of healing done per point of mana by a spell
--]]
local function getHPMana(spell, healing, spellCrit)
	local hpcast = getHPCast(spell, healing, spellCrit);
	local manaCost = HealPointsSpells:getManaCost(spell, spellCrit);
	return hpcast / manaCost;
end

local function getHPTotalRegen(spell, healing, spellCrit, spellHaste, manaRegenCasting, mana)
	local hpPerCast = getHPCast(spell, healing, spellCrit);
	local manaCost = HealPointsSpells:getManaCost(spell, spellCrit);

	local interval;
	if (spell.class['type'] == 'regrowth') then
		interval = HealPointsSpells:GetCastTime(spell, spellHaste) + HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval'];
	elseif (spell.class['type'] == 'hot' or spell.class['type'] == 'lifebloom') then
		interval = HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval'];
	elseif (spell.class['type'] == 'instant') then
		interval = math.max(getGCD(spellHaste), spell.class['cooldown']);
	else
		interval = math.max(getGCD(spellHaste), HealPointsSpells:GetCastTime(spell, spellHaste)); 
	end

	local manaUsageSpell = manaCost - interval * (manaRegenCasting / 5);
	local hpPerMana = hpPerCast / manaUsageSpell;
	local hpPerManaBar = hpPerMana * mana;
	
	local numberCasts = mana / manaUsageSpell;
	local totalTime = numberCasts * interval;

	return hpPerManaBar, totalTime;
end

--[[
	Calculate the total amount of healing that can be done by a spell in the specified amount of time, starting at the specified mana level
	@param spell - A SPELLTABLE entry of the spell to calculate
	@param healing - A number that is an amount of bonus healing
	@param seconds - How long we run the calculation for
	@param startMana - The player's starting mana
	@param spellCrit - The player's chance of getting a critical heal
	@param spellHaste - The player's spell haste
	@param manaRegenCasting - The players mana regeneration per second while in combat
	@param manaRegenNormal - The players mana regeneration per second while out of combat
--]]
local function computeTotalHealingNormal(spell, healing, seconds, startMana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal)
	--DEFAULT_CHAT_FRAME:AddMessage("Computing total healing for "..spell.class['name'].."("..spell['rank']..") secs="..seconds.." mana="..startMana,1,1,1);

	assert(spell);
	assert(healing);
	assert(seconds);
	assert(startMana);
	assert(spellCrit);
	assert(spellHaste);
	assert(manaRegenCasting);
	assert(manaRegenNormal);

	local hpCast = getHPCast(spell, healing, spellCrit);
	local castDelay = math.max(getGCD(spellHaste), HealPointsSpells:GetCastTime(spell, spellHaste)); 
	if (spell.class['cooldown'] ~= nil) then
		castDelay = math.max(castDelay, spell.class['cooldown']);
	end
	local manaCost = HealPointsSpells:getManaCost(spell, spellCrit);
	local hps = hpCast / castDelay;

	-- Phase 1: Has mana to chain-cast
	local manaUsageSecond = (manaCost / castDelay) - (manaRegenCasting / 5);
	local secondsPhase1 = math.min(seconds, startMana / manaUsageSecond);
	local totalHealed = secondsPhase1 * hps;

	-- Phase 2: Has no mana left
	local secondsPhase2 = seconds - secondsPhase1;
	if (secondsPhase2 > 0) then
		local cycleTime; -- Seconds between each casting
		if (manaRegenCasting > manaCost) then -- chaincast inside 5s rule
			local secs = manaCost / (manaRegenCasting / 5);
			cycleTime = math.max(secs, castDelay);
		else
			local manaToRegen = manaCost - manaRegenCasting;
			cycleTime = 5 + manaToRegen / (manaRegenNormal / 5);
		end
		local spellsCast = secondsPhase2 / cycleTime;
		local healedPhase2 = spellsCast * hpCast;
		totalHealed = totalHealed + healedPhase2;
	end

	return totalHealed;
end

local function computeTotalHealingRegrowth(spell, healing, seconds, startMana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal)
	assert(spell);
	assert(healing);
	assert(seconds);
	assert(startMana);
	assert(spellCrit);
	assert(spellHaste);
	assert(manaRegenCasting);
	assert(manaRegenNormal);

	--DEFAULT_CHAT_FRAME:AddMessage("Computing total healing for "..spell.class['name'].."("..spell['rank']..") secs="..seconds.." mana="..startMana,1,1,1);
	local manaCost = HealPointsSpells:getManaCost(spell, spellCrit);
	local castTime = math.max(getGCD(spellHaste), HealPointsSpells:GetCastTime(spell, spellHaste)); 
	local percentBonus = HealPointsSpells:getHealingPercentBonus(spell);
	local healFactorBurst, healFactorHot = HealPointsSpells:getHealingFactor(spell);
	local numtargets = HealPoints.db.char.hot['numtargets'];
	spellCrit = spellCrit + HealPointsSpells:getSpellCritBonus(spell);
	healing = healing + HealPointsSpells:getHealingBonus(spell);

	local avgBurst = (((spell['min'] + spell['max']) / 2) + healing * healFactorBurst) * percentBonus * (1 + spellCrit / 200);
	local healTick = (spell['tick'] + healing * healFactorHot / spell.class['tickcount']) * percentBonus;

	-- Phase 1: Has mana to chain-cast
	local manaUsageSecond = (manaCost / castTime) - (manaRegenCasting / 5);
	local secondsPhase1 = math.min(seconds, startMana / manaUsageSecond);
	local secsBetweenSameTarget = math.max(castTime, numtargets * castTime);
	local ticsPerCast = secsBetweenSameTarget / spell.class['interval'];
	ticsPerCast = math.min(ticsPerCast, HealPointsSpells:getNumberOfTicks(spell));
	local spellsCast = secondsPhase1 / castTime;

	local totalHealed = spellsCast * (avgBurst + ticsPerCast * healTick);

	-- Phase 2: Has no mana left
	local secondsPhase2 = seconds - secondsPhase1;
	if (secondsPhase2 > 0) then
		local cycleTime; -- Seconds between each casting
		if (manaRegenCasting > manaCost) then -- chaincast inside 5s rule
			local secs = manaCost / (manaRegenCasting / 5);
			cycleTime = math.max(secs, castTime);
		else
			local manaToRegen = manaCost - manaRegenCasting;
			cycleTime = 5 + manaToRegen / (manaRegenNormal / 5);
		end
		secsBetweenSameTarget = math.max(cycleTime, numtargets * cycleTime);
		ticsPerCast = secsBetweenSameTarget / spell.class['interval'];
		ticsPerCast = math.min(ticsPerCast, HealPointsSpells:getNumberOfTicks(spell));
		spellsCast = secondsPhase2 / cycleTime;

		local healedPhase2 = spellsCast * (avgBurst + ticsPerCast * healTick);
		totalHealed = totalHealed + healedPhase2;
	end

	return totalHealed;
end

local function computeTotalHealingLifebloom(spell, healing, seconds, startMana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal)

	--Get the the percent bonus to healing spells (e.g. 1.07 = 7%)
	local percentBonus = HealPointsSpells:getHealingPercentBonus(spell);

	local healFactorHot, healFactorBurst = HealPointsSpells:getHealingFactor(spell);


	spellCrit = spellCrit + HealPointsSpells:getSpellCritBonus(spell);
	healing = healing + HealPointsSpells:getHealingBonus(spell);
	
	local avgBurst = (
			spell['burst'] 
			--+ (healing + HealPointsBS:GetBonus('AVG_BURST_LIFEBL')) * healFactorBurst
		) * percentBonus;
		
	local healTick = (
		spell['tick'] 
		--+ (healing + HealPointsBS:GetBonus('AVG_HOT_LIFEBL')) * healFactorHot / spell.class['tickcount']
		) * percentBonus;
	avgBurst = avgBurst * (1 + spellCrit / 200);
	local duration = HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval'];
	local manaCost = HealPointsSpells:getManaCost(spell, spellCrit);
	local numtargets = HealPoints.db.char.hot['numtargets'];

	if (_G["HealPointsOptionsConfigLifeBl"]:GetChecked() == 1) then -- stack lifeblooms
		-- Phase 1: Has enough mana/regen to keep stacking lifeblooms up on numtargets constantly
		numtargets = math.min(3, numtargets); -- max 3 targets
		local cycleTime = duration;
		local manaUsageCycle = numtargets * manaCost;
		local regenTimeCasting = math.min(cycleTime, 5 + (numtargets - 1) * getGCD(spellHaste));
		local regenTimeNormal = math.max(0, cycleTime - regenTimeCasting);
		local manaRegainCycle = regenTimeCasting * (manaRegenCasting / 5) + regenTimeNormal * (manaRegenNormal / 5);
		local numCycles = seconds / cycleTime;
		if (manaRegainCycle < manaUsageCycle) then -- if you regen more than you use, you'll never go oom.
			numCycles = math.min(numCycles, startMana / (manaUsageCycle - manaRegainCycle));
		end
		local secondsPhase1 = numCycles * cycleTime;
		local numTicks = numCycles * numtargets * 3 * (HealPointsSpells:getNumberOfTicks(spell) - 1); -- Overwrites before the last tick
		if (numCycles > 2) then -- Ticks lost due to building stacks
			numTicks = numTicks - numtargets * 3 * (HealPointsSpells:getNumberOfTicks(spell) - 1) - 3 * 0.75 * (numtargets * numtargets - numtargets);
		elseif (numCycles == 2) then
			numTicks = numTicks - numtargets * 3 * (HealPointsSpells:getNumberOfTicks(spell) - 1) - 2 * 0.75 * (numtargets * numtargets - numtargets);
		elseif (numCycles == 1) then
			numTicks = numTicks - numtargets * 2 * (HealPointsSpells:getNumberOfTicks(spell) - 1) - 0.75 * (numtargets * numtargets - numtargets);
		end
		local totalHealed = numTicks * healTick;
		if (numCycles > 1) then
			totalHealed = totalHealed + numtargets * (avgBurst + healTick); -- Last tick + burst when oom
		end

		-- Phase 2: Has no mana left, no stacking
		local secondsPhase2 = seconds - secondsPhase1;
		if (secondsPhase2 > 0) then
			local manaToRegen = manaUsageCycle - regenTimeCasting * (manaRegenCasting / 5);
			cycleTime = regenTimeCasting + manaToRegen / (manaRegenNormal / 5);
			numCycles = secondsPhase2 / cycleTime;
			local healedPhase2 = numCycles * numtargets * (avgBurst + healTick * HealPointsSpells:getNumberOfTicks(spell));
			totalHealed = totalHealed + healedPhase2;
		end
		return totalHealed;
	
	else
		-- Phase 1: Has enough mana/regen to keep lifebloom up on numtargets constantly
		local cycleTime = math.max(duration + 1, numtargets * getGCD(spellHaste)); -- time between casts on the same target
		local manaUsageCycle = numtargets * manaCost;
		local regenTimeCasting = math.min(cycleTime, 5 + (numtargets - 1) * getGCD(spellHaste));
		local regenTimeNormal = math.max(0, cycleTime - regenTimeCasting);
		local manaRegainCycle = regenTimeCasting * (manaRegenCasting / 5) + regenTimeNormal * (manaRegenNormal / 5);
		local healingPerCycle = numtargets * (avgBurst + healTick * HealPointsSpells:getNumberOfTicks(spell));
		local numCycles = seconds / cycleTime;
		if (manaRegainCycle < manaUsageCycle) then -- if you regen more than you use, you'll never go oom.
			numCycles = math.min(numCycles, startMana / (manaUsageCycle - manaRegainCycle));
		end
		local secondsPhase1 = numCycles * cycleTime;
		local totalHealed = numCycles * healingPerCycle;

		-- Phase 2: Has no mana left
		local secondsPhase2 = seconds - secondsPhase1;
		if (secondsPhase2 > 0) then
			local manaToRegen = manaUsageCycle - regenTimeCasting * (manaRegenCasting / 5);
			cycleTime = regenTimeCasting + manaToRegen / (manaRegenNormal / 5);
			numCycles = secondsPhase2 / cycleTime;
			local healedPhase2 = numCycles * healingPerCycle;
			totalHealed = totalHealed + healedPhase2;
		end

		return totalHealed;
	end
end

local function computeTotalHealingHot(spell, healing, seconds, startMana, spelllHaste, manaRegenCasting, manaRegenNormal)
	local hpCast = getHPCast(spell, healing, 0); -- Can't crit
	local duration = HealPointsSpells:getNumberOfTicks(spell) * spell.class['interval'];
	local manaCost = HealPointsSpells:getManaCost(spell, 0); -- Can't crit
	local numtargets = HealPoints.db.char.hot['numtargets'];

	-- Phase 1: Has enough mana/regen to keep the HoT up on numtargets constantly
	local cycleTime = math.max(duration + 1, numtargets * getGCD(spellHaste)); -- time between casts on the same target
	local manaUsageCycle = numtargets * manaCost;
	local regenTimeCasting = math.min(cycleTime, 5 + (numtargets - 1) * getGCD(spellHaste));
	local regenTimeNormal = math.max(0, cycleTime - regenTimeCasting);
	local manaRegainCycle = regenTimeCasting * (manaRegenCasting / 5) + regenTimeNormal * (manaRegenNormal / 5);
	local healingPerCycle = numtargets * hpCast;
	local numCycles = seconds / cycleTime;
	if (manaRegainCycle < manaUsageCycle) then -- if you regen more than you use, you'll never go oom.
		numCycles = math.min(numCycles, startMana / (manaUsageCycle - manaRegainCycle));
	end
	local secondsPhase1 = numCycles * cycleTime;
	local totalHealed = numCycles * healingPerCycle;

	-- Phase 2: Has no mana left
	local secondsPhase2 = seconds - secondsPhase1;
	if (secondsPhase2 > 0) then
		local manaToRegen = manaUsageCycle - regenTimeCasting * (manaRegenCasting / 5);
		cycleTime = regenTimeCasting + manaToRegen / (manaRegenNormal / 5);
		numCycles = secondsPhase2 / cycleTime;
		local healedPhase2 = numCycles * healingPerCycle;
		totalHealed = totalHealed + healedPhase2;
	end

	return totalHealed;
end

--[[
	Caluclates one of the two terms in the HealPoints equation (HealPoints = PowerPoints+EndurancePoints
	Calculates how much healing can be done by an entire mana bar; takes mana regen into account
	@param maxMana: number - The player's current percent [0..100] of their mana
--]]
local function computePowerPoints(healing, spellCrit, spellHaste, maxMana, manaRegenCasting, manaRegenNormal)

	assert(healing, "computePowerPoints - healing is nil");
	assert(spellCrit, "computePowerPoints - spellCrit is nil");
	assert(spellHaste, "computePowerPoints - spellHaste is nil");
	assert(maxMana, "computePowerPoints - maxMana is nil");
	assert(manaRegenCasting, "computePowerPoints - manaRegenCasting is nil");
	assert(manaRegenNormal, "computePowerPoints - manaRegenNormal is nil");

	local mana = HealPoints.db.char.power['mana'] * maxMana / 100;
	if (HealPoints.db.char.power['auto'] == true) then
		local powerfulSpell;
		local maxPowerPoints = 0;
		for i = 1, table.getn(HealPointsSpells.SPELLTABLE) do
			for j = 1, table.getn(HealPointsSpells.SPELLTABLE[i]), 1 do
				local spell = HealPointsSpells.SPELLTABLE[i][j];
				if (spell ~= nil and HealPointsSpells:getHighestSpellRank(spell.class.classID) == spell['rank'] and spell.class['type'] ~= 'instant') then
					local powerPoints;
					if (spell.class['type'] == 'hot') then
						powerPoints = computeTotalHealingHot(spell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellHaste, manaRegenCasting, manaRegenNormal);
					elseif (spell.class['type'] == 'regrowth') then
						powerPoints = computeTotalHealingRegrowth(spell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					elseif (spell.class['type'] == 'lifebloom') then
						powerPoints = computeTotalHealingLifebloom(spell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					else
						powerPoints = computeTotalHealingNormal(spell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					end
					if (powerPoints > maxPowerPoints) then
						powerfulSpell = spell;
						maxPowerPoints = powerPoints;
					end
				end
			end
		end
		return maxPowerPoints, powerfulSpell;

	else
		local powerfulSpell = HealPointsSpells:getSpell(HealPoints.db.char.power['spell']);
		local powerPoints;
		if (powerfulSpell.class['type'] == 'hot') then
			powerPoints = computeTotalHealingHot(powerfulSpell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellHaste, manaRegenCasting, manaRegenNormal);
		elseif (powerfulSpell.class['type'] == 'regrowth') then
			powerPoints = computeTotalHealingRegrowth(powerfulSpell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		elseif (powerfulSpell.class['type'] == 'lifebloom') then
			powerPoints = computeTotalHealingLifebloom(powerfulSpell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		else
			powerPoints = computeTotalHealingNormal(powerfulSpell, healing, 60 * HealPoints.db.char.power['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		end
		return powerPoints, powerfulSpell;
	end
end

--[[
	Caluclates one of the two terms in the HealPoints equation (HealPoints = PowerPoints+EndurancePoints
	Calculates how much healing can be done by an entire mana bar; takes mana regen into account
	@param maxMana: percent - The player's current mana (0..100), typically 0%
--]]
local function computeEndurancePoints(healing, spellCrit, spellHaste, maxMana, manaRegenCasting, manaRegenNormal)

	local mana = HealPoints.db.char.endurance['mana'] * maxMana / 100;
	if (HealPoints.db.char.endurance['auto'] == true) then
		local efficientSpell;
		local maxEndurancePoints = 0;
		for i = 1, table.getn(HealPointsSpells.SPELLTABLE), 1 do
			for j = 1, table.getn(HealPointsSpells.SPELLTABLE[i]), 1 do
				local spell = HealPointsSpells.SPELLTABLE[i][j];
				if (spell ~= nil and HealPointsSpells:getHighestSpellRank(spell.class.classID) == spell['rank'] and spell.class['type'] ~= 'instant') then
					local endurancePoints;
					if (spell.class['type'] == 'hot') then
						endurancePoints = computeTotalHealingHot(spell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellHaste, manaRegenCasting, manaRegenNormal);
					elseif (spell.class['type'] == 'regrowth') then
						endurancePoints = computeTotalHealingRegrowth(spell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					elseif (spell.class['type'] == 'lifebloom') then
						endurancePoints = computeTotalHealingLifebloom(spell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					else
						endurancePoints = computeTotalHealingNormal(spell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
					end
					if (endurancePoints > maxEndurancePoints) then
						efficientSpell = spell;
						maxEndurancePoints = endurancePoints;
					end
				end
			end
		end
		return maxEndurancePoints, efficientSpell;

	else
		local efficientSpell = HealPointsSpells:getSpell(HealPoints.db.char.endurance['spell']);
		local endurancePoints;
		if (efficientSpell.class['type'] == 'hot') then
			endurancePoints = computeTotalHealingHot(efficientSpell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellHaste, manaRegenCasting, manaRegenNormal);
		elseif (efficientSpell.class['type'] == 'regrowth') then
			endurancePoints = computeTotalHealingRegrowth(efficientSpell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		elseif (efficientSpell.class['type'] == 'lifebloom') then
			endurancePoints = computeTotalHealingLifebloom(efficientSpell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		else
			endurancePoints = computeTotalHealingNormal(efficientSpell, healing, 60 * HealPoints.db.char.endurance['duration'], mana, spellCrit, spellHaste, manaRegenCasting, manaRegenNormal);
		end
		return endurancePoints, efficientSpell;
	end
end

--[[
	Calculates the player's
		healPoints, powerPoints, endurancePoints
	as well as the most powerfulSpell and the most efficient spell
	@param maxMana: number - The players current mana as a percent [0..100]. 
--]]
local function computeHealpoints(healing, spellCrit, spellHaste, maxMana, manaRegenCasting, manaRegenNormal)
	
	assert(healing, "computePowerPoints - healing is nil");
	assert(spellCrit, "computePowerPoints - spellCrit is nil");
	assert(spellHaste, "computePowerPoints - spellHaste is nil");
	assert(maxMana, "computePowerPoints - maxMana is nil");
	assert(manaRegenCasting, "computePowerPoints - manaRegenCasting is nil");
	assert(manaRegenNormal, "computePowerPoints - manaRegenNormal is nil");

	local powerPoints, powerfulSpell = computePowerPoints(healing, spellCrit, spellHaste, maxMana, manaRegenCasting, manaRegenNormal);
	local endurancePoints, efficientSpell = computeEndurancePoints(healing, spellCrit, spellHaste, maxMana, manaRegenCasting, manaRegenNormal);

	local healPoints = powerPoints + endurancePoints;

	return healPoints, powerPoints, endurancePoints, powerfulSpell, efficientSpell;
end

--[[Compute current and new values of Mana Regeneration
	Returns
		manaRegenNormal
		manaRegenCasting
		manaRegenNormalNew
		manaRegenCastingNew
		percentRegen
--]]
local function getCurrentAndNewManaRegen_Depricated(itemIntDelta, itemSpiDelta, itemCastingRegenDelta, itemManaDelta)
	-- Base stats  
	local _, intellect, _, _ = UnitStat("player", 4);
	local _, spirit, _, _ = UnitStat("player", 5);

	local spiritRegen = StatLogic:GetNormalManaRegenFromSpi(spirit + itemSpiDelta, intellect + itemIntDelta);
	
	--Mana regeneration from Spirit while in combat increased by xxx percent 
	--	e.g. Paladin Holy specialization Meditation "Allows 50% of your mana regeneration from Spirit to continue while in combat"
	local percentRegen = StatLogic:GetNormalManaRegenFromSpi(spirit, intellect);
	
		local castingRegen = HealPointsUtil:GetBaseCombatManaRegen() --base combat regen amount that everyone always gets
			+ itemCastingRegenDelta; --items can no longer have casting regen (removed from game). But i can leave param here
  
	local manaRegenNormal = castingRegen + spiritRegen;
	local manaRegenCasting = castingRegen + percentRegen*spiritRegen;
  
	-- Calculator bonuses
	local calcInt = _G["HealPointsVariables1InputBox"]:GetNumber() * (1 + BonusPercentInt);
	local calcSpi = _G["HealPointsVariables2InputBox"]:GetNumber() * (1 + BonusPercentSpi);
	local calcMana = _G["HealPointsVariables3InputBox"]:GetNumber();
	local calcCastingRegen = _G["HealPointsVariables4InputBox"]:GetNumber();
	local calcNormalRegen = _G["HealPointsVariables5InputBox"]:GetNumber();
	local calcPercentRegen = _G["HealPointsVariables6InputBox"]:GetNumber() / 100;

	local spiritRegenNew = StatLogic:GetNormalManaRegenFromSpi(spirit + itemSpiDelta + calcSpi, intellect + itemIntDelta + calcInt);
	
	local manaRegenNormalNew = castingRegen + spiritRegenNew;
	local manaRegenCastingNew = castingRegen + percentRegen*spiritRegenNew;

	if (_G["HealPointsOptionsConfigReplenish"]:GetChecked() == 1) then -- add replenishment
		local currentMana = UnitManaMax("player") + itemManaDelta + (15 * itemIntDelta);
		local replenishmp5 = 0.0125 * currentMana; -- 0.25% every sec = 1.25% every 5 sec
		manaRegenNormal = manaRegenNormal + replenishmp5;
		manaRegenCasting = manaRegenCasting + replenishmp5;

		local newMana = UnitManaMax("player") + itemManaDelta + (15 * itemIntDelta) + calcMana + 15 * calcInt;
		local replenishmp5New = 0.0125 * newMana; -- 0.25% every sec = 1.25% every 5 sec
		manaRegenNormalNew = manaRegenNormalNew + replenishmp5New;
		manaRegenCastingNew = manaRegenCastingNew + replenishmp5New;
	end

	if (_G["HealPointsOptionsConfigDivinePlea"]:GetChecked() == 1) then -- add divine plea
		local currentMana = UnitManaMax("player") + itemManaDelta + (15 * itemIntDelta);
		local replenishmp5 = 0.0208333 * currentMana; -- 25% every minute = 2.083% every 5 sec
		manaRegenNormal = manaRegenNormal + replenishmp5;
		manaRegenCasting = manaRegenCasting + replenishmp5;

		local newMana = UnitManaMax("player") + itemManaDelta + (15 * itemIntDelta) + calcMana + 15 * calcInt;
		local replenishmp5New = 0.0208333 * newMana; -- 25% every minute = 2.083% every 5 sec
		manaRegenNormalNew = manaRegenNormalNew + replenishmp5New;
		manaRegenCastingNew = manaRegenCastingNew + replenishmp5New;
	end

	return manaRegenNormal, manaRegenCasting, manaRegenNormalNew, manaRegenCastingNew, percentRegen;
end

local function getCurrentAndNewMaxMana_depricated(itemManaDelta, itemIntDelta)
	local currentMana = UnitManaMax("player") + itemManaDelta + (15 * itemIntDelta);

	local intDelta = _G["HealPointsVariables1InputBox"]:GetNumber() * (1 + BonusPercentInt);
	local newMana = currentMana + (_G["HealPointsVariables3InputBox"]:GetNumber() + 15 * intDelta);
	return currentMana, newMana;
end

local function getCurrentAndNewHealing_depricated(itemHealDelta, itemIntDelta, itemSpiDelta, itemAgiDelta)
	local intBonus = 0;
	local spiBonus = 0;
	local agiBonus = 0;
  
	local _, className = UnitClass("player");
	if (className == "PALADIN") then
		intBonus = 0.04 * HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.HOLYGU); -- 4/8/12/16/20% of intellect
	elseif (className == "PRIEST") then
		spiBonus = 0.05 * HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.SPIGUI); -- 5/10/15/20/25% of spirit
	elseif (className == "DRUID") then
		intBonus = 0.04 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.LUNGUI); -- 4/8/12% of intellect
		agiBonus = 0.35 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NURINS); -- 35/70% of agility
		if (HealPointsUtil:isPlayerBuffUp(HealPointsLoc.BUFF_TREELI)) then
		spiBonus = 0.05 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.IMPTOL); -- 5/10% of spirit while in Tree Form
		end 
	elseif (className == "SHAMAN") then
		intBonus = 0.05 * HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.NATBLE); -- 5/10/15% of intellect
	end
	intBonus = intBonus; --+ 0.01 * HealPointsBS:GetBonus('HEALFROMINT');

	local healingDelta = intBonus * itemIntDelta + spiBonus * itemSpiDelta + agiBonus * itemAgiDelta;

	local healingDeltaNew = _G["HealPointsVariables7InputBox"]:GetNumber();
	itemIntDelta = itemIntDelta + _G["HealPointsVariables1InputBox"]:GetNumber() * (1 + BonusPercentInt); 
	itemSpiDelta = itemSpiDelta + _G["HealPointsVariables2InputBox"]:GetNumber() * (1 + BonusPercentSpi);
	healingDeltaNew = healingDeltaNew + intBonus * itemIntDelta + spiBonus * itemSpiDelta + agiBonus * itemAgiDelta;

	local currentHealing = GetSpellBonusHealing() + itemHealDelta;

	return currentHealing + healingDelta, currentHealing + healingDeltaNew;
end

local function getCurrentAndNewSpellCrit_depricated(itemIntDelta, itemCritRatingDelta)
	local spellCrit = 0;

	local _, className = UnitClass("player");
	if (className == "PALADIN" or className == "PRIEST") then
		spellCrit = GetSpellCritChance(2); -- Holy
	elseif (className == "DRUID" or className == "SHAMAN") then
		spellCrit = GetSpellCritChance(4); -- Nature
	end

	local critFromDeltaRating = StatLogic:GetEffectFromRating(itemCritRatingDelta, CR_CRIT_SPELL);
	local critFromDeltaInt = StatLogic:GetSpellCritFromInt(itemIntDelta);
	local currentSpellCrit = spellCrit + critFromDeltaRating + critFromDeltaInt;

	local critRatingDelta = itemCritRatingDelta + _G["HealPointsVariables8InputBox"]:GetNumber();
	local intDelta = itemIntDelta + _G["HealPointsVariables1InputBox"]:GetNumber() * (1 + BonusPercentInt);

	critFromDeltaRating = StatLogic:GetEffectFromRating(critRatingDelta, CR_CRIT_SPELL);
	critFromDeltaInt = StatLogic:GetSpellCritFromInt(intDelta);
	local newSpellCrit = spellCrit + critFromDeltaRating + critFromDeltaInt;

	return currentSpellCrit, newSpellCrit;
end

local function getCurrentAndNewSpellHaste_depricated(itemHasteRatingDelta)
	local spellHaste = GetCombatRatingBonus(CR_HASTE_SPELL);

	local hasteFromDelta = StatLogic:GetEffectFromRating(itemHasteRatingDelta, CR_HASTE_SPELL);
	local currentSpellHaste = spellHaste + hasteFromDelta;

	hasteFromDelta = StatLogic:GetEffectFromRating(itemHasteRatingDelta + _G["HealPointsVariables9InputBox"]:GetNumber(), CR_HASTE_SPELL);
	local newSpellHaste = spellHaste + hasteFromDelta;
  
	return currentSpellHaste, newSpellHaste;
end

local function displaySpellStats_depricated(spell) -- Show derived stats about specified spell
	if (BonusPercentInt == nil) then
		self:talentsChanged();
	end

	local _, manaRegenCasting, _, manaRegenCastingNew = getCurrentAndNewManaRegen(0, 0, 0, 0);
	local currentMana, newMana = getCurrentAndNewMaxMana(0, 0);
	local currentHealing, newHealing = getCurrentAndNewHealing(0, 0, 0, 0);
	local currentSpellCrit, newSpellCrit = getCurrentAndNewSpellCrit(0, 0);
	local currentSpellHaste, newSpellHaste = getCurrentAndNewSpellHaste(0);

	local oldHPSec = getHPSec(spell, currentHealing, currentSpellCrit, currentSpellHaste);
	local newHPSec = getHPSec(spell, newHealing, newSpellCrit, newSpellHaste);
	local oldHPMana = getHPMana(spell, currentHealing, currentSpellCrit);
	local newHPMana = getHPMana(spell, newHealing, newSpellCrit);
	local oldHPCast = getHPCast(spell, currentHealing, currentSpellCrit);
	local newHPCast = getHPCast(spell, newHealing, newSpellCrit);
	local oldHPTotalRegen, t1 = getHPTotalRegen(spell, currentHealing, currentSpellCrit, currentSpellHaste, manaRegenCasting, currentMana);
	local newHPTotalRegen, t2 = getHPTotalRegen(spell, newHealing, newSpellCrit, newSpellHaste, manaRegenCastingNew, newMana);
	local oldCrit = currentSpellCrit + HealPointsSpells:getSpellCritBonus(spell);
	local newCrit = newSpellCrit + HealPointsSpells:getSpellCritBonus(spell);
	local oldCastTime = HealPointsSpells:GetCastTime(spell, currentSpellHaste);
	local newCastTime = HealPointsSpells:GetCastTime(spell, newSpellHaste);

--[[
	-- Common stats
	HealPointsCalculatorUI:setSpellStat(1, "HP/s:", oldHPSec, newHPSec, "%d");
	HealPointsCalculatorUI:setSpellStat(2, "HP/mana:", oldHPMana, newHPMana, "%1.1f");
	HealPointsCalculatorUI:setSpellStat(3, "HP/cast:", oldHPCast, newHPCast, "%d");
	HealPointsCalculatorUI:setSpellStat(4, "HP/manabar:", oldHPTotalRegen, newHPTotalRegen, "%d");

	-- Spell specific stats
	if (spell['class'].type ~= 'hot') then
		HealPointsCalculatorUI:setSpellStat(5, "Spell crit(%):", oldCrit, newCrit, "%1.2f");
	else
		local oldHPTick = getHPTick(spell, currentHealing);
		local newHPTick = getHPTick(spell, newHealing);
		HealPointsCalculatorUI:setSpellStat(5, "HP/tick:", oldHPTick, newHPTick, "%d");
	end

	if (spell['class'].type ~= 'hot' and spell['class'].type ~= 'instant' and spell['class'].type ~= 'lifebloom' and
		spell.class.classID ~= HealPointsSpells.SPELL_HOLYNOVA and spell.class.classID ~= HealPointsSpells.SPELL_COH) then
		HealPointsCalculatorUI:setSpellStat(6, "Cast time:", oldCastTime, newCastTime, "%1.2f");
	elseif (spell['class'].type == 'lifebloom') then
		local oldHPTick = getHPTick(spell, currentHealing);
		local newHPTick = getHPTick(spell, newHealing);
		HealPointsCalculatorUI:setSpellStat(6, "HP/tick:", oldHPTick, newHPTick, "%d");
	else
		HealPointsCalculatorUI:setSpellStat(6, nil);
	end

	if (spell['class'].type == 'regrowth') then
		local oldHPTick = getHPTick(spell, currentHealing);
		local newHPTick = getHPTick(spell, newHealing);
		HealPointsCalculatorUI:setSpellStat(7, "HP/tick:", oldHPTick, newHPTick, "%d");
	else
		HealPointsCalculatorUI:setSpellStat(7, nil);
	end

	local healFactor1, healFactor2 = HealPointsSpells:getHealingFactor(spell);
	local spellBonus = HealPointsSpells:getHealingPercentBonus(spell);
	healFactor1 = healFactor1 * spellBonus;
	if (healFactor2 ~= nil) then
		healFactor2 = healFactor2 * spellBonus;
	end
	if (spell['class'].type == 'aoe') then
		healFactor1 = healFactor1 * math.min(5, HealPoints.db.char.hot['numtargets']);
	end
	HealPointsCalculatorUI:setHealFactor(spell, healFactor1, healFactor2);

	HealPointsCalculatorUI:setTimeToOOM(t1, t2);
--]]
end

--[[
function HealPointsCalculator:updateHealPoints() -- Update the healpoints statistic
	if (BonusPercentInt == nil) then
		self:talentsChanged();
	end

	local manaRegenNormal, manaRegenCasting, manaRegenNormalNew, manaRegenCastingNew
	local currentMana, newMana
	local currentHealing, newHealing
	local currentSpellCrit, newSpellCrit
	local currentSpellHaste, newSpellHaste
	
	local healPointsCurrent, healedPowerful, healedEfficient, powerfulCurrent, efficientCurrent =
			computeHealpoints(currentHealing, currentSpellCrit, currentSpellHaste, currentMana, manaRegenCasting, manaRegenNormal);

	local healPointsNew, healedPowerfulNew, healedEfficientNew, powerfulNew, efficientNew =
			computeHealpoints(newHealing, newSpellCrit, newSpellHaste, newMana, manaRegenCastingNew, manaRegenNormalNew);

	self.HP_LastPowerfulCurrent = powerfulCurrent;
	self.HP_LastPowerfulNew = powerfulNew;
	self.HP_LastEfficientCurrent = efficientCurrent;
	self.HP_LastEfficientNew = efficientNew;

	if (HealPoints_CalcFrame:IsVisible()) then
		HealPointsCalculatorUI:setHealPoints(healedPowerful, healedPowerfulNew, healedEfficient, healedEfficientNew);
	end
	if (CharacterFrame:IsVisible()) then
		HealPointsCharUI:setStats(healPointsCurrent, healedPowerful, healedEfficient, powerfulCurrent, efficientCurrent);
	end
end
--]]

--[[
	Calculate the HealPoints gained per point of attributes
			intellect
			spirit
			spell power
			crit rating
			haste rating
	returns
		intellectValue
		spiritValue
		spellPowerValue
		critRatingValue
		hasteRatingValue
--]]
function HealPointsCalculator:ComputeHealPointsDelta(currentStats)
	--local currentStats = HealPointsCalculator.GetCurre7ntHealPoints();
	assert(currentStats, "HealPointsCalculator:ComputeHealPointsDelta - currentStats cannot be nil")

	local iv, sv, spv, scrv, shrv;

	local newStats;

	--1 intellect
	local changes = { intellect = 1 };
	newStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(newStats);
	iv = (newStats.healPoints - currentStats.healPoints);

	--1 spirit
	local changes = { spirit = 1 };
	newStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(newStats);
	sv = (newStats.healPoints - currentStats.healPoints);

	--1 spell power
	local changes = { spellPower = 1 };
	newStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(newStats);
	spv = (newStats.healPoints - currentStats.healPoints);

	--1 spell crit rating
	local changes = { spellCritRating = 1 };
	newStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(newStats);
	scrv = (newStats.healPoints - currentStats.healPoints);

	--1 spell haste rating
	local changes = { spellHasteRating = 1 };
	newStats = HealPointsCalculator:ApplyChanges(currentStats, changes);
	HealPointsCalculator:CalculateHealPoints(newStats);
	shrv = (newStats.healPoints - currentStats.healPoints);

	return iv, sv, spv, scrv, shrv, currentStats.powerfulSpell, currentStats.efficientSpell;
end

local function computeHealpointsDiff_Depricated(itemLink)
	local function lookup(table, index)
		if (table == nil or table[index] == nil) then
			return 0;
		end
		return table[index];
	end

	local function computeHP(itemBonuses, slotBonuses)
		local intDelta = HealPointsUtil:round((lookup(itemBonuses, "INT") - lookup(slotBonuses, "INT")) * (1 + BonusPercentInt));
		local spiDelta = HealPointsUtil:round((lookup(itemBonuses, "SPI") - lookup(slotBonuses, "SPI")) * (1 + BonusPercentSpi));
		local manaDelta = HealPointsUtil:round(lookup(itemBonuses, "MANA") - lookup(slotBonuses, "MANA"));
		local agiDelta = lookup(itemBonuses, "AGI") - lookup(slotBonuses, "AGI");
		local critRatingDelta = lookup(itemBonuses, "SPELLCRITRATING") - lookup(slotBonuses, "SPELLCRITRATING");
		local hasteRatingDelta = lookup(itemBonuses, "SPELLHASTERATING") - lookup(slotBonuses, "SPELLHASTERATING");
		local castingRegenDelta = lookup(itemBonuses, "MANAREG") - lookup(slotBonuses, "MANAREG");
		local healDelta = lookup(itemBonuses, "HEAL") - lookup(slotBonuses, "HEAL");

		local iHealing, iHealingNew = getCurrentAndNewHealing(healDelta, intDelta, spiDelta, agiDelta);
		local iSpellCrit, iSpellCritNew = getCurrentAndNewSpellCrit(intDelta, critRatingDelta);
		local iSpellHaste, iSpellHasteNew = getCurrentAndNewSpellHaste(hasteRatingDelta);
		local iMana, iManaNew = getCurrentAndNewMaxMana(manaDelta, intDelta);
		local iManaRegenNormal, iManaRegenCasting, iManaRegenNormalNew, iManaRegenCastingNew = getCurrentAndNewManaRegen(intDelta, spiDelta, castingRegenDelta, manaDelta);

		iHealing = HealPointsUtil:round(iHealing);
		iMana = HealPointsUtil:round(iMana);
		iHealingNew = HealPointsUtil:round(iHealingNew);
		iManaNew = HealPointsUtil:round(iManaNew);

		if (HealPoints.db.char.debug ~= 0) then
			DEFAULT_CHAT_FRAME:AddMessage("Tooltip debug: +int:"..intDelta.. " +spi:"..spiDelta.." +healing:"..healDelta.." +mana/5:"..castingRegenDelta.." +critRating:"..critRatingDelta.." +hasteRating:"..hasteRatingDelta,0.6,0.6,1);
			DEFAULT_CHAT_FRAME:AddMessage("New stats: "..iHealing..", "..iSpellCrit..", "..iSpellHaste..", "..iMana..", "..iManaRegenCasting..", "..iManaRegenNormal,0.6,0.6,1);
		end
		local iHealPoints = computeHealpoints(iHealing, iSpellCrit, iSpellHaste, iMana, iManaRegenCasting, iManaRegenNormal);
		local iHealPointsNew = iHealPoints;
		if (iHealingNew ~= iHealing or iSpellCritNew ~= iSpellCrit or iManaNew ~= iMana or iManaRegenCastingNew ~= iManaRegenCasting or
			iManaRegenNormalNew ~= iManaRegenNormal or iSpellHasteNew ~= iSpellHaste) then
			iHealPointsNew = computeHealpoints(iHealingNew, iSpellCritNew, iSpellHasteNew, iManaNew, iManaRegenCastingNew, iManaRegenNormalNew);
		end
		return iHealPoints, iHealPointsNew
	end

	if (BonusPercentInt == nil) then
		self:talentsChanged();
	end

	local itemBonuses, slotBonuses, slotBonuses2; -- = HealPointsBS:getItemSlotBonuses(itemLink);
	if (itemBonuses == nil) then
		return 0, 0;
	end

	if (HealPoints.db.char.debug ~= 0) then
		DEFAULT_CHAT_FRAME:AddMessage("Bonuses for: "..itemLink,1,1,1);
		for bonus, details in pairs(itemBonuses) do
			HealPoints:Print(bonus..": "..details);
		end
		if (slotBonuses ~= nil) then
			DEFAULT_CHAT_FRAME:AddMessage("Replaces these bonuses: ",1,1,1);
			for bonus, details in pairs(slotBonuses) do
			HealPoints:Print(bonus..": "..details);
			end
		end
	end

	-- Current healpoints
	local healing, healingNew= getCurrentAndNewHealing(0, 0, 0, 0);
	local spellCrit, spellCritNew = getCurrentAndNewSpellCrit(0, 0);
	local spellHaste, spellHasteNew = getCurrentAndNewSpellHaste(0);
	local mana, manaNew = getCurrentAndNewMaxMana(0, 0)
	local manaRegenNormal, manaRegenCasting, manaRegenNormalNew, manaRegenCastingNew = getCurrentAndNewManaRegen(0, 0, 0, 0);

	--DEFAULT_CHAT_FRAME:AddMessage("Current: "..healing..", "..spellCrit..", "..spellHaste..", "..mana..", "..manaRegenCasting..", "..manaRegenNormal,0.6,0.6,1);
	local healPoints = computeHealpoints(healing, spellCrit, spellHaste, mana, manaRegenCasting, manaRegenNormal);
	local healPointsNew = healPoints;
	if (healingNew ~= healing or spellCritNew ~= spellCrit or manaNew ~= mana or manaRegenCastingNew ~= manaRegenCasting or manaRegenNormalNew ~= manaRegenNormal) then
		healPointsNew = computeHealpoints(healingNew, spellCritNew, spellHasteNew, manaNew, manaRegenCastingNew, manaRegenNormalNew);
	end

	-- Difference
	local iHealPoints, iHealPointsNew = computeHP(itemBonuses, slotBonuses);
	if (slotBonuses2) then
		local iHealPoints2, iHealPointsNew2 = computeHP(itemBonuses, slotBonuses2);
		return iHealPoints - healPoints, iHealPointsNew - healPointsNew, iHealPoints2 - healPoints, iHealPointsNew2 - healPointsNew;
	else
		return iHealPoints - healPoints, iHealPointsNew - healPointsNew;
	end
end

local function talentsChanged_Depricated()
	BonusPercentInt = 0;
	BonusPercentSpi = 0;

	local _, raceName = UnitRace("player");
	if (raceName == "Human") then
		BonusPercentSpi = 0.03;
	end

	local _, className = UnitClass("player");
	if (className == "PALADIN") then
		local divineIntellect = HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.DIVINT); -- Increases intellect by 3% per rank
		BonusPercentInt = 0.03 * divineIntellect;
	elseif (className == "PRIEST") then

		local mentalStrength = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.MENSTR); -- Increases intellect by 3% per rank
		BonusPercentInt = 0.03 * mentalStrength;

		local enlightenment = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.ENLIGH); -- Increases spi by 1% per rank
		BonusPercentSpi = BonusPercentSpi + 0.01 * enlightenment;

		local spiritOfRedemption = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.SPIRED); -- Increases spi by 5%
		BonusPercentSpi = BonusPercentSpi + 0.05 * spiritOfRedemption;

	elseif (className == "DRUID") then
		local heartOfTheWild = HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.HEOFWI); -- Increases int by 4% per rank
		BonusPercentInt = 0.04 * heartOfTheWild;

		local livingSpirit = HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.LIVSPI); -- Increases spi by 5% per rank
		BonusPercentSpi = 0.05 * livingSpirit;

		local survivalFittest = HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.SURFIT); -- Increases all stats by 2% per rank
		BonusPercentInt = BonusPercentInt + 0.02 * survivalFittest;
		BonusPercentSpi = BonusPercentSpi + 0.02 * survivalFittest;

	elseif (className == "SHAMAN") then
		local ancestralKnowledge = HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.ANCKNO); -- Increases int by 2% per rank
		BonusPercentInt = BonusPercentInt + 0.02 * ancestralKnowledge;
	end
end

--[[
	Get a table of the player's current (important) stats:
	
			data.playerLevel  		e.g. 85
			data.playerClass  		e.g. "Paladin"
			data.strength
			data.agility
			data.stamina
			data.intellect
			data.spirit
			data.mana				Player's maximum mana
			data.baseManaRegen 		Player's "base" mana regeneration (i.e. tooltip "Combat Regen")

			data.spellPower			Players tooltip "Spell Power" e.g. 3138
			data.spellCritChance	Player's tooltip "Crit Chance" e.g. 16.04%
			data.spellCritRating 	Player's spell "Crit Rating" e.g. Crit rating 263 (+4.36% crit chance) 
			data.spellHaste			Player's spell "Haste" (e.g. Haste +11.31%)
			data.spellHasteRating	Player's spell "Haste Rating" e.g. Haste rating 487 (+11.31% Haste)
			
			data.attackPower		Player's attack power, e.g. 399 (Increases damage with melee weapons by 28.5 damage per second.)
--]]
function HealPointsCalculator:GetCurrentPlayerStats(data)
	--assert(data)
	
	local result = data;
	if (result == nil) then
		result = {};
	end;

	local unit = "player"
	
	result.playerLevel = UnitLevel(unit)   --WoW API
--	result.playerHealth = UnitHealthMax(unit)   --WoW API
	result.playerClass = select(2, UnitClass(unit))	--   WoW API
	
	--Attributes
	--local stat, effectiveStat, posBuff, negBuff = UnitStat(unit, 1); --1=Strength
	result.strength =  UnitStat(unit, 1); --1=Strength   WoW API
	result.agility =   UnitStat(unit, 2); --2=Agility   WoW API
	result.stamina =   UnitStat(unit, 3); --3=Stamina   WoW API
	result.intellect = UnitStat(unit, 4); --4=Intellect   WoW API
	result.spirit =    UnitStat(unit, 5); --5=Spirit   WoW API
	
	-- Mana
	result.mana = UnitManaMax(unit); --WoW API
	result.baseMana = HealPointsUtil:GetBaseMana();

	local manaRegen, combatRegen = GetManaRegen(); --while not in combat, while in combat
	result.normalManaRegen = manaRegen;
	result.baseManaRegen = HealPointsUtil:GetBaseCombatManaRegen();
	--assert(result.baseManaRegen ~= nil);
	result.combatManaRegen = combatRegen;
	result.combatManaRegenPercent = 0;
	result.manaCostModifier = 1.0;
	
	-- Spell
	result.spellPower = GetSpellBonusHealing(); --WoW API GetSpellBonusHealing (e.g. 3138)
	result.spellCritRating = GetCombatRating(CR_CRIT_SPELL); --WoW API GetCombatRating(CR_CRIT_SPELL) = 263 ("Crit rating 263 (+4.36% crit chance)")
	--result.spellCritChanceBonus = GetCombatRatingBonus(CR_CRIT_SPELL); --WoW API GetCombatRatingBonus(CR_CRIT_SPELL) = 4.36 ("Crit rating 263 (+4.36% crit chance)")
	result.spellCritChance = GetSpellCritChance(2); --2=Holy  WoW API GetSpellCritChance(2) = 16.038 ("Crit Chance 16.04%")
	
	result.spellHasteRating = GetCombatRating(CR_HASTE_SPELL); --WoW API GetCombatRating(CR_HASTE_SPELL) = 487 ("Haste rating 487 (+11.31% haste)")
	local spellHasteBonus = GetCombatRatingBonus(CR_HASTE_SPELL);  --WoW API GetCombatRatingBonus(CR_HASTE_SPELL) = 11.3108 ("Haste rating 487 (+11.31% haste)")
	result.spellHaste = spellHasteBonus; --haste might come from other sources

	-- Melee
	result.attackPower = UnitAttackPower("unit"); --WoW API e.g. UnitAttackPower("player") = 399
	
	--[[
	-- Mana regen (casting, normal, %casting)
	local manaRegenNormal, manaRegenCasting, manaRegenNormalNew, manaRegenCastingNew, percentRegen = getCurrentAndNewManaRegen(0, 0, 0, 0);
	local manaRegenCastingDelta = manaRegenCastingNew - manaRegenCasting - _G["HealPointsVariables4InputBox"]:GetNumber();
	local manaRegenNormalDelta = manaRegenNormalNew - manaRegenNormal - _G["HealPointsVariables5InputBox"]:GetNumber();
	HealPointsCalculatorUI:setVariableStatInfo(4, manaRegenCasting, manaRegenCastingDelta, "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfo(5, manaRegenNormal, manaRegenNormalDelta, "%5.0f");
	HealPointsCalculatorUI:setVariableStatInfo(6, 100 * percentRegen, 0, "%5.0f");
	--]]

	-- Healing spell power
	--local currentHealing, newHealing = getCurrentAndNewHealing(0, 0, 0, 0);
	--HealPointsCalculatorUI:setVariableStatInfo(7, currentHealing, newHealing - currentHealing - _G["HealPointsVariables7InputBox"):GetNumber(), "%5.0f"];

	-- Spell crit rating
	--HealPointsCalculatorUI:setVariableStatInfo(8, GetCombatRating(11), 0, "%5.0f");

	-- Spell haste rating
	--HealPointsCalculatorUI:setVariableStatInfo(9, GetCombatRating(20), 0, "%5.0f");
	
	return result;
end;		


--[[
	Calculate result = playerStats+changes
		Changes can come from either
			a tooltip link -> HealPointsTooltips:GetChangesTable -> ApplyChanges
			the calculator -> HealPointsCalculatorUI:GetChanges -> ApplyChanges
		@param playerStats:table - a table of the player's stats to which the changes will be applied
		@param changes:table - a table of changes that will be applied to playerStats
	@returns: table - a player stats table containing the new player stats
--]]	
function HealPointsCalculator:ApplyChanges(playerStats, changes)
	assert(playerStats, "HealPointsCalculator:ApplyChanges - playerStats is nil");
	assert(changes, "HealPointsCalculator:ApplyChanges - changes is nil");

	--Get everything source has
	local results = Toolkit:CopyTable(playerStats);
	
	--[[
		data.playerLevel  		e.g. 85
		data.playerClass  		e.g. "Paladin"
		
		data.strength
		data.agility
		data.stamina
		data.intellect
		data.spirit
		
		data.mana				Player's maximum mana
		data.combatManaRegen 		Player's "base" mana regeneration (i.e. tooltip "Combat Regen")

		data.spellPower			Players tooltip "Spell Power" e.g. 3138
		data.spellCritRating 	Player's spell "Crit Rating" e.g. Crit rating 263 (+4.36% crit chance) 
		data.spellCritChance	Player's tooltip "Crit Chance" e.g. 16.04%
		data.spellHaste			Player's spell "Haste" (e.g. Haste +11.31%)
		data.spellHasteRating	Player's spell "Haste Rating" e.g. Haste rating 487 (+11.31% Haste)
			
		data.attackPower		Player's attack power, e.g. 399 (Increases damage with melee weapons by 28.5 damage per second.)
	--]]
	
	if (changes.strength ~= nil) then
		results.strength = playerStats.strength + (changes.strength or 0);
		changes.spellPower = (changes.spellPower or 0) + StatLogic:GetStatMod("ADD_HEAL_MOD_STR")*changes.strength;
		
		changes.strength = nil; --remove item
	end;
	
	changes.attackPower = (changes.attackPower or 0);
	if (changes.attackPower ~= nil) then
		results.attackPower = playerStats.attackPower + changes.attackPower;
		changes.spellPower = (changes.spellPower or 0) + StatLogic:GetStatMod("ADD_HEAL_MOD_AP")*changes.attackPower;

		changes.attackPower = nil; --remove item
	end;
	
	if (changes.agility ~= nil) then
		results.agility = playerStats.agility + changes.agility;
		changes.spellPower = (changes.spellPower or 0) + StatLogic:GetStatMod("ADD_HEAL_MOD_AGI")*changes.agility;

		changes.agility = nil; --remove item
	end;

	--results.stamina = playerStats.stamina + (changes.stamina or 0);

	local recalculateManaRegen = false;

	if (changes.intellect ~= nil) then
		results.intellect = playerStats.intellect + changes.intellect;

		changes.mana = (changes.mana or 0) + HealPointsUtil:GetManaFromInt(changes.intellect);
		changes.spellPower = (changes.spellPower or 0) + HealPointsUtil:GetSpellPowerFromInt(changes.intellect);
		changes.spellCritChance = (changes.spellCritChance or 0) + StatLogic:GetSpellCritFromInt(changes.intellect);

		recalculateManaRegen = true; --we changed our intellect, we have to recompute mana regen
		changes.intellect = nil; --remove item
	end;

	if (changes.spirit ~= nil) then
		results.spirit = playerStats.spirit + changes.spirit;

		recalculateManaRegen = true; --we changed our spirit, we have to recompute mana regen

		changes.spirit = nil; --remove item
	end;
	
	--local modMana = StatLogic:GetStatMod("MOD_MANA");
	--if (modMana ~= 0) then
	--end;
	
	if (changes.mana ~= nil) then
		results.mana = playerStats.mana + changes.mana;
		changes.spellPower = (changes.spellPower or 0) + StatLogic:GetStatMod("ADD_HEAL_MOD_MANA")*changes.mana;
		--changes.combatManaRegen = (changes.combatManaRegen or 0) + StatLogic:GetStatMod("ADD_COMBAT_MANA_REGEN_MOD_MANA")*changes.mana;
		--changes.combatManaRegen = (changes.combatManaRegen or 0)+ StatLogic:GetStatMod("ADD_COMBAT_MANA_REGEN_MOD_MANA_REGEN")*changes.mana;

		recalculateManaRegen = true;
		changes.mana = nil;
	end;

	--HealPoints:Print("results.mana: "..results.mana);

	--MOD_MANA

	if (recalculateManaRegen) then
		local baseManaRegen = HealPointsUtil:GetBaseCombatManaRegen(); --per second
		--HealPoints:Print("baseManaRegen: "..baseManaRegen);
		local manaRegenThroughSpirit = StatLogic:GetNormalManaRegenFromSpi(results.spirit, results.intellect)/5;
		
		local manaRegenBefore = playerStats.normalManaRegen;
		local manaRegenAfter = baseManaRegen + manaRegenThroughSpirit;

		--HealPoints:Print("manaRegenBefore: "..manaRegenBefore);
		--HealPoints:Print("manaRegenAfter: "..manaRegenAfter);

		changes.normalManaRegen = (manaRegenAfter - manaRegenBefore);

		local combatRegenBefore = playerStats.combatManaRegen;
		local combatRegenAfter = baseManaRegen + StatLogic:GetStatMod("ADD_COMBAT_MANA_REGEN_MOD_MANA_REGEN")*manaRegenThroughSpirit;
		changes.combatManaRegen = (combatRegenAfter - combatRegenBefore);
	end;

	
	if (changes.combatManaRegen ~= nil) then
		results.combatManaRegen = playerStats.combatManaRegen + changes.combatManaRegen;
		changes.combatManaRegen = nil;
	--	HealPoints:Print("results.combatManaRegin: "..results.combatManaRegen);
	end;

	if (changes.normalManaRegen ~= nil) then	
		results.normalManaRegen = playerStats.normalManaRegen + changes.normalManaRegen; 
		changes.normalManaRegen = nil;
		--HealPoints:Print("results.normalManaRegen: "..results.normalManaRegen);
	end;

	if (changes.combatManaRegenPercent ~= nil) then
--		results.combatManaRegenPercent = playerStats.combatManaRegenPercent + changes.combatManaRegenPercent;
		changes.combatManaRegenPercent = nil;
	end;

	if (changes.manaCost ~= nil) then
		results.manaCostModifier = playerStats.manaCostModifier + changes.manaCost;
		changes.manaCost = nil;
	end;
	
	--HealPoints:Print("changes.spellPower: "..changes.spellPower);
	if (changes.spellPower ~= nil) then
		results.spellPower = playerStats.spellPower + changes.spellPower;
		changes.spellPower = nil;
	end;

	if (changes.spellCritRating ~= nil) then
		results.spellCritRating = playerStats.spellCritRating + changes.spellCritRating;
		changes.spellCritChance = (changes.spellCritChance or 0) + StatLogic:GetEffectFromRating(changes.spellCritRating, CR_CRIT_SPELL)/100.0;

		changes.spellCritRating = nil;
	end;

	if (changes.spellCritChance ~= nil) then
		results.spellCritChance = playerStats.spellCritChance + (changes.spellCritChance or 0);

		changes.spellCritChance = nil;
	end;

	if (changes.spellHasteRating ~= nil) then
		results.spellHasteRating = (playerStats.spellHasteRating or 0) + changes.spellHasteRating;

		changes.spellHasteRating = nil;

		results.spellHaste = StatLogic:GetEffectFromRating(results.spellHasteRating, CR_HASTE_SPELL);
	end;

	--Remove items from changes table that i know i can't use
	--results.MASTERY_RATING = nil;
	results.SPELL_DMG = nil;
	results.itemType = nil;
	results.link = nil;
	results.diffItemType1 = nil;
	results.diffItemType2 = nil;
	results.EXPERTISE_RATING = nil;
	results.ARMOR = nil;
	results.MELEE_CRIT_RATING = nil;
	results.MELEE_HASTE_RATING = nil;
	results.MELEE_HIT_RATING = nil;
	results.SPELL_HIT_RATING = nil;
	results.EMPTY_SOCKET_META = nil;
	results.EMPTY_SOCKET_RED = nil;
	results.EMPTY_SOCKET_YELLOW = nil;
	results.EMPTY_SOCKET_BLUE = nil;
	results.MAX_DAMAGE = nil;
	results.DPS = nil;
	results.DODGE_RATING = nil;
	results.PARRY_RATING = nil;
	results.ARMOR_BONUS = nil;
	results.SHADOW_RES = nil;
	results.FIRE_RES = nil;
	results.FROST_RES = nil;

	--Remove items from changes table that i know i can't use
	changes.MASTERY_RATING = nil;
	changes.SPELL_DMG = nil;
	changes.itemType = nil;
	changes.link = nil;
	changes.diffItemType1 = nil;
	changes.diffItemType2 = nil;
	changes.EXPERTISE_RATING = nil;
	changes.ARMOR = nil;
	changes.MELEE_CRIT_RATING = nil;
	changes.MELEE_HASTE_RATING = nil;
	changes.MELEE_HIT_RATING = nil;
	changes.SPELL_HIT_RATING = nil;
	changes.EMPTY_SOCKET_META = nil;
	changes.EMPTY_SOCKET_RED = nil;
	changes.EMPTY_SOCKET_YELLOW = nil;
	changes.EMPTY_SOCKET_BLUE = nil;
	changes.MAX_DAMAGE = nil;
	changes.DPS = nil;
	changes.DODGE_RATING = nil;
	changes.PARRY_RATING = nil;
	changes.ARMOR_BONUS = nil;
	changes.SHADOW_RES = nil;
	changes.FIRE_RES = nil;
	changes.FROST_RES = nil;


	--HealPoints:Print("changes.length"..Toolkit:VarAsString(table.maxn(changes)));
	if next(changes) ~= nil then
		HealPoints:Print("HealPointsCalculator:ApplyChanges - changes leftover: "..Toolkit:VarAsString(changes));
	end;

	return results;
end;


--[[
	Based on the stats contained in passed playerStats, calculate the 
			HealPoints, PowerPoints and EndurancePoints, and other stats
		and write them to the same supplied playerStats object:
			- healPoints
			- powerPoints
			- endurancePoints
			- hpSec - (HP/second) Health Points healed per second by the selected spell
			- hpMana - (HP/mana) Health Points healed per point of mana
			- hpCast - (HP/cast) Health Points healed per cast of the selected spell
--]]
function HealPointsCalculator:CalculateHealPoints(playerStats, forceSpell)
	assert(playerStats, "HealPointsCalculator:CalculateHealPoints: Argument 'playerStats' is nil");
	--assert(forceSpell, "HealPointsCalculator:CalculateHealPoints - Argument 'spell' is nil")
	
	--i need the spell in order to determine the points; the hp/sec, hp/cast, etc all come from the spell.
	local manaRegenNormal = playerStats.normalManaRegen;
	local manaRegenCasting = playerStats.combatManaRegen;

	local currentMana = playerStats.mana;
	local currentHealing = playerStats.spellPower;
	local currentSpellCrit = playerStats.spellCritChance;
	local currentSpellHaste = playerStats.spellHaste;

	--HealPoints:Print("HealPointsCalculator:CalculateHealPoints");

	assert(currentHealing, "CalculateHealPoints - currentHealing is nil");
	assert(currentSpellCrit, "CalculateHealPoints - currentSpellCrit is nil");
	assert(currentSpellHaste, "CalculateHealPoints - currentSpellHaste is nil");
	assert(currentMana, "CalculateHealPoints - currentMana is nil");
	assert(manaRegenCasting, "CalculateHealPoints - manaRegenCasting is nil");
	assert(manaRegenNormal, "CalculateHealPoints - manaRegenNormal is nil");
	
	local healPoints, powerPoints, endurancePoints, powerfulSpell, efficientSpell =
			computeHealpoints(currentHealing, currentSpellCrit, currentSpellHaste, currentMana, manaRegenCasting, manaRegenNormal, forceSpell);

	playerStats.powerfulSpell = powerfulSpell;
	playerStats.efficientSpell = efficientSpell;
	
	self.HP_LastPowerfulCurrent = powerfulSpell;
--	self.HP_LastPowerfulNew = powerfulNew;
	self.HP_LastEfficientCurrent = efficientSpell;
--	self.HP_LastEfficientNew = efficientNew;

	--HealPoints:Print("CalculateHealPoints - powerfulSpell="..Toolkit:VarAsString(powerfulSpell));
	--HealPoints:Print("CalculateHealPoints - efficientSpell="..Toolkit:VarAsString(efficientSpell));

--	if (HealPoints_CalcFrame:IsVisible()) then
		--HealPointsCalculatorUI:setHealPoints(healedPowerful, healedPowerfulNew, healedEfficient, healedEfficientNew);
	--end
	--if (CharacterFrame:IsVisible()) then
--		HealPointsCharUI:setStats(healPointsCurrent, healedPowerful, healedEfficient, powerfulCurrent, efficientCurrent);
--	end
	
	if (forceSpell) then
		playerStats.hpSec = getHPSec(forceSpell, currentHealing, currentSpellCrit, currentSpellHaste);
		playerStats.hpMana = getHPMana(forceSpell, currentHealing, currentSpellCrit);
		playerStats.hpCast = getHPCast(forceSpell, currentHealing, currentSpellCrit);
		playerStats.hpManaBar = getHPTotalRegen(forceSpell, currentHealing, currentSpellCrit, currentSpellHaste, manaRegenCasting, currentMana);
	end;

	playerStats.powerPoints = powerPoints;
	playerStats.endurancePoints = endurancePoints;
	playerStats.healPoints = healPoints;
end;

--[[
	Returns an object representing the players current stats and healpoint calculation results
	Note: This object might be calculated on the fly, or might be the cached result of a recent calculation
--]]
function HealPointsCalculator:GetCurrentHealPoints(selectedSpell)
	local currentStats = HealPointsCalculator:GetCurrentPlayerStats();

	HealPointsCalculator:CalculateHealPoints(currentStats, selectedSpell);

	return currentStats;
end;