local Toolkit = LibStub:GetLibrary("Toolkit-1.0");

HealPointsSpells = {
	SPELLTABLE = { },

--[[
	SPELLTABLE.class.type
			normal, instant: all healing performed in one chunk     
					man, max: range of the heal
					computeTotalHealingNormal

			aoe: hits a number of people
					min, max: range of the heal
					computeTotalHealingNormal

			hot: healing done by number of ticks
					uses computeTotalHealingHot()
					tick: amount healed by individual tick
					tickcount: the total number of ticks

			regrowth: initial burst but a HOT component
					min, max:
					tick:
					tickcount:
					interval: interval between hot ticks
					computeTotalHealingRegrowth

			chain:
					min, max: range of the heal
	

			lifebloom: healing done over time, with a final possible burst
					uses computeTotalHealingLifebloom
					has a hot part:
						tick: amount healed by individual tick
					has a burst part
						burst: amount healed by the burst
--]]

	--Paladin
	SPELL_FOL = 1,          --Paladin: Flash of Light
	SPELL_HL = 2,           --Paladin: Holy Light
	SPELL_DivineLight = 3,  --Paladin: Divine Light
	SPELL_HR = 4,           --Paladin: Holy Radiance
	SPELL_HS = 5,           --Paladin: Holy Shock

	SPELL_FH = 11,
	SPELL_LH = 12,
	SPELL_HEAL = 13,
	SPELL_GH = 14,
	SPELL_RENEW = 15,
	SPELL_POH = 16,
	SPELL_HOLYNOVA = 17,
	SPELL_COH = 18,

	SPELL_REGR = 21,
	SPELL_HT = 22,
	SPELL_REJUV = 23,
	SPELL_LIFEBL = 24,

	--Shaman
	SPELL_HealingSurge = 31, --Shaman: Healing Surge (8004) 
	SPELL_HW = 32,           --Shaman: Healing Wave (331)
	SPELL_GHW = 33,          --Shaman: Greater Healing Wave (77472)
	SPELL_CHAIN = 34,        --Shaman: Chain Heal
	SPELL_Riptide = 35,      --Shaman: Riptide (61295)
};

local rankCache = { };

function HealPointsSpells:init()
	local _, className = UnitClass("player");
	if (className == "PALADIN") then
	
		--Paladin: Flash of Light
		local fol = { classID = HealPointsSpells.SPELL_FOL, type = 'normal', abbr = "FoL", time = 1.5, coeff = 0.80571};
		self.SPELLTABLE[1] = {{ class=fol, id=19750, rank=1, min=6907, max=7749, manaP=31, level=16 }};

		--Paladin: Holy Light
		local hl = { classID = HealPointsSpells.SPELL_HL, type = 'normal', abbr = "HL", time = 3.0, coeff = 1.6114};
		self.SPELLTABLE[2] = {{ class=hl, id=635, rank=1, min=4163, max=4637, manaP=12, level=7 }};

		--Paladin: Divine Light
		local dl = { classID = HealPointsSpells.SPELL_DivineLight, type='normal', abbr="DL", time=3.0, coeff=1.6114};
		self.SPELLTABLE[3] = {{class=dl, id=82326, rank=1, min=11100, max=12366, manaP=35, level=62 }};

		--Paladin: Holy Shock
		hs = { classID = HealPointsSpells.SPELL_HS, type = 'instant', abbr = "HS", cooldown = 6, coeff = 0.80571};
		self.SPELLTABLE[4] = {{ class=hs, id=20473, rank=1, min=2629, max=2847, manaP=7, level=10 }};
					
		--Paladin: Holy Radiance
		hl = { classID = HealPointsSpells.SPELL_HR, type='regrowth', abbr="HR", time=3.0, tickcount=3, interval=1, coeff1=1.6114, coeff2=0.376};
		self.SPELLTABLE[5] = {{ class=hl, id=82327, rank=1, min=2623, max=3205, tick=493, manaP=35, level=83 }};
					

	elseif (className == "PRIEST") then -- Missing: Binding heal, Penance, Prayer of mending, Holy Word: Sanctuary, Holy Word: Serenity
	
		--Priest: Flash Heal
		fh = { classID = HealPointsSpells.SPELL_FH, type = 'normal', abbr = "FH", time = 1.5, coeff = 0.8068};
		self.SPELLTABLE[1] = {{ class=fh, id=2061, rank=1, min=152, max=176, manaP=28, level=3 }};

		--Priest: Heal
		heal =   { classID = HealPointsSpells.SPELL_HEAL, type = 'normal', abbr = "Heal" };
		self.SPELLTABLE[2] = { { class = heal, id = 2050, rank = 1, min = 83, max = 95, time = 3.0, manaP = 9, level = 16 }}

		--Priest: Greater Heal
		gh =   { classID = HealPointsSpells.SPELL_GH, type = 'normal', abbr = "GH", time = 3.0, coeff = 1.6135};
		self.SPELLTABLE[3] = {{ class = heal, id =  2060, rank = 1, min = 658, max = 764, manaP = 27, level = 38 }};

		--Priest: Renew
		renew = { classID = HealPointsSpells.SPELL_RENEW, type = 'hot', abbr = "Renew", tickcount = 5, interval = 3, coeff = 1.88};
		self.SPELLTABLE[4] = { { class = renew, id = 139, rank = 1, tick = 30, manaP = 17, level = 8 }};

		--Priest: Prayer of Healing
		poh = { classID = HealPointsSpells.SPELL_POH, type = 'aoe', abbr = "PoH", time = 3, coeff = 0.8068};
		self.SPELLTABLE[5] = { { class = poh, id = 596, rank = 1, min =	310, max = 326, manaP =	26, level = 44 }};

		--Priest: Holy Nova
		holynova = { classID = HealPointsSpells.SPELL_HOLYNOVA, type = 'aoe', abbr = "HNova", time = 0, coeff = 0.3035};
		self.SPELLTABLE[6] = { { class = holynova, id = 15237, rank = 1, min =	110,  max =	126, manaP =15, level = 62 }};
							
		--Priest: Circle of Healing
		coh = { classID = HealPointsSpells.SPELL_COH, type = 'aoe', abbr = 'CoH', time = 0, coeff = 0.4020, cooldown = 10};
		self.SPELLTABLE[7] = { { class = coh, id = 34861, rank = 1, min = 802, max = 886, manaP = 21, level = 59 }};

	elseif (className == "DRUID") then -- Missing: Wild Growth, Nourish
	
		--Druid: Regrowth
		regr = { classID = HealPointsSpells.SPELL_REGR, type = 'regrowth', abbr = "Regr", tickcount = 7, interval = 3, time = 2, coeff1 = 0.539, coeff2 = 1.316};
		self.SPELLTABLE[1] = { { class = regr, id = 8936, rank = 1, min = 104, max = 116, tick = 14, manaP = 33, level = 12 }};
						
		--Druid: Healing Touch
		ht = { classID = HealPointsSpells.SPELL_HT, type = 'normal', abbr = "HT", coeff = 1.6104};
		self.SPELLTABLE[2] = { { class = ht, id = 5185, rank = 1, min = 5885, max = 6949, time = 3.0, manaP = 30, level = 78 }};
		
		--Druid: Rejuvenation
		rejuv = { classID = HealPointsSpells.SPELL_REJUV, type = 'hot', abbr = "Rejuv", tickcount = 4, interval = 3, coeff = 1.88};
		self.SPELLTABLE[3] = { { class = rejuv, id = 774, rank = 1, tick = 35, manaP = 20, level = 3 }};

		--Druid: Lifebloom
		lifebl = { classID = HealPointsSpells.SPELL_LIFEBL, type = 'lifebloom', abbr = "LifeBl", tickcount = 7, interval = 1, coeff1 = 0.666, coeff2 = 0.6453};
		self.SPELLTABLE[4] = { { class = lifebl, id = 33763, rank = 1, burst = 697, tick = 86, manaP = 7, level = 64 }};

	
	elseif (className == "SHAMAN") then
	
		--Shaman: Healing Surge
		lhw = { classID = HealPointsSpells.SPELL_HealingSurge, type = 'normal', abbr = "HS", time = 1.5, coeff=0.80571};
		self.SPELLTABLE[1] = { { class = lhw, id =	 8004, rank = 1, min = 4484, max = 5122, manaP = 27, level = 20 }};
						
		--Shaman: Healing Wave
		hw = { classID = HealPointsSpells.SPELL_HW, type = 'normal', abbr = "HW", time = 3.0, coeff=1.6114};
		self.SPELLTABLE[2] = { { class = hw, id  = 331, rank =  1, min = 2803, max = 3201, manaP = 9, level =  7 }};

		--Shaman: Greater Healing Wave
		ghw = { classID = HealPointsSpells.SPELL_GHW, type = 'normal', abbr = "GHW", time = 3.0, coeff=1.6114};
		self.SPELLTABLE[3] = { { class = ghw, id = 77472, rank =  1, min = 8968, max = 10244, manaP = 33, level =  68 }};

		--Shaman: Riptide
		local rt = { classID = HealPointsSpells.SPELL_Riptide, type = 'regrowth', abbr = "RipT", tickcount=5, interval=3, time=0, coeff1=0.666, coeff2=0.6453};
		self.SPELLTABLE[4] = { { class = rt, id = 61295, rank =  1, min = 2364, max = 5590, tick=1117, manaP = 10, level =  69 }};

		--Shaman: Chain Heal
		chain = { classID = HealPointsSpells.SPELL_CHAIN, type = 'chain', abbr = "CH", time = 2.5, coeff = 1.34}; 
		self.SPELLTABLE[5] = { { class = chain, id = 1064, rank = 1, min = 3283, max = 3749, manaP = 20, level = 40 }};
	end
end

function HealPointsSpells:GetSpellLink(spell, linkable)
	if (spell == nil) then
		return "";
	end;

	local spellName = GetSpellInfo(spell.id);
	local s;

	if (linkable) then
		s = string.format("|cff71d5ff|Hspell:%d|h[%s]|h|r", spell.id, spellName);
	else
		s = string.format("|cff71d5ff%s|r", spellName);
	end;

	return s;
end;



function HealPointsSpells.GetSpellClassIDs()
	local _, className = UnitClass("player");

	local result;

	if (className == "PALADIN") then
		result = { 
				HealPointsSpells.SPELL_FOL,         --Flash of Light
				HealPointsSpells.SPELL_HL,          --Holy Light
				HealPointsSpells.SPELL_DivineLight, --Divine Light
				HealPointsSpells.SPELL_HR,          --Holy Radiance
				HealPointsSpells.SPELL_HS };        --Holy Shock
	elseif (className == "PRIEST") then
		result = { 
				HealPointsSpells.SPELL_FH, 
				HealPointsSpells.SPELL_LH, 
				HealPointsSpells.SPELL_HEAL,
				HealPointsSpells.SPELL_GH, 
				HealPointsSpells.SPELL_RENEW, 
				HealPointsSpells.SPELL_POH,
				HealPointsSpells.SPELL_HOLYNOVA, 
				HealPointsSpells.SPELL_COH };
	elseif (className == "DRUID") then
		result = { 
				HealPointsSpells.SPELL_HT, 
				HealPointsSpells.SPELL_REJUV, 
				HealPointsSpells.SPELL_REGR,
				HealPointsSpells.SPELL_LIFEBL };
	elseif (className == "SHAMAN") then
		result = { 
				HealPointsSpells.SPELL_HealingSurge, 
				HealPointsSpells.SPELL_HW, 
				HealPointsSpells.SPELL_GHW, 
				HealPointsSpells.SPELL_Riptide,
				HealPointsSpells.SPELL_CHAIN };
	else
		result = {};
	end

	return result;
end;

function HealPointsSpells:getSpell(spellName) -- TODO
	for t = 1, table.getn(self.SPELLTABLE), 1 do
		for i = 1, table.getn(self.SPELLTABLE[t]), 1 do
			local thisName = GetSpellInfo(self.SPELLTABLE[t][i]['id']);
			if (thisName == spellName and self.SPELLTABLE[t][i]['rank'] == self:getHighestSpellRank(self.SPELLTABLE[t][i]['class']['classID'])) then
				return self.SPELLTABLE[t][i];
			end
		end
	end
end

--[[
	Return a spell-table entry by the spell's ClassID
	e.g. SPELL_FOL, SPELL_HL, SPELL_FH
--]]
function HealPointsSpells:getSpellByClassID(spellClassID) 
	for t = 1, table.getn(self.SPELLTABLE), 1 do
		for i = 1, table.getn(self.SPELLTABLE[t]), 1 do
			if (self.SPELLTABLE[t][i]['class']['classID'] == spellClassID and self.SPELLTABLE[t][i]['rank'] == self:getHighestSpellRank(self.SPELLTABLE[t][i]['class']['classID'])) then
				return self.SPELLTABLE[t][i];
			end
		end
	end
end

function HealPointsSpells:spellsChanged()
	for k in pairs(rankCache) do
		rankCache[k] = nil;
	end
end

--[[
	Returns the cast time, in seconds, of a spell
	@param spell An table entry from HealPointsSpells:SPELLTABLE that indicates the spell to calculate the cast time of
	@param spellHaste A number percentage of the current spell haste
	@returns A number contianing the cast time of the selected spell
	@example 
			local hl = HealPointsSpells:GetSpellByName("Holy Light");
	        local castTime = HealPointsSpell:GetCastTime(hl, 2.06); --2.06=2.06%
--]]
function HealPointsSpells:GetCastTime(spell, spellHaste) 
	assert(spell, "HealPointsSpells:GetCastTime - spell is nil");
	assert(spellHaste, "HealPointsSpells:GetCastTime - spellHaste is nil");

	--HealPoints:Print("HealPointsSpells:GetCastTime(spell, spellHaste)");
	--HealPoints:Print("    spell = "..Toolkit:VarAsString(spell));
	--HealPoints:Print("    spellHaste = "..Toolkit:VarAsString(spellHaste));

	local castTime = spell.class['time'];
	if (castTime == nil) then
		castTime = spell['time'];
	end

	local _, className = UnitClass("player");

	if (className == "PRIEST") then
		if (spell.class.classID == HealPointsSpells.SPELL_HEAL or spell.class.classID == HealPointsSpells.SPELL_GH) then
			castTime = castTime - HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.DIVFUR) * 0.1; -- Reduced time for Heal/Greater Heal by 0.1 per rank
		end

	elseif (className == "DRUID") then
		if (spell.class.classID == HealPointsSpells.SPELL_HT) then
			castTime = castTime - HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NATURA) * 0.1; -- Reduced time for Healing Touch by 0.1 per rank
		end

	elseif (className == "SHAMAN") then
		if (spell.class.classID == HealPointsSpells.SPELL_HW or spell.class.classID == HealPointsSpells.SPELL_GHW) then
			--Purification (Restoration Specialization) (spellID 16213)
			--	Increases the effectiveness of your 
			--	healing spells by 25%, and reduces the 
			--  casting time of your Healing Wave and
			--  Greater Healing Wave by 0.5 sec.
			-- 16213
			if (IsSpellKnown(16213)) then
				castTime = castTime - 0.5;
			end; 
		end
	end
	
	if (castTime ~= nil) then
		local newCastTime = castTime / (1 + spellHaste / 100);
		--HealPoints:Print("HealPointsSpell:GetCastTime - reducing cast time ("..castTime..") by "..spellHaste.." to "..newCastTime);
		castTime = newCastTime;
	end
	
	return castTime;
end

function HealPointsSpells:getNumberOfTicks(spell)
	local tickcount = spell.class['tickcount'];

	local _, className = UnitClass("player");

	if (className == "PRIEST") then
		if (spell.class.classID == HealPointsSpells.SPELL_RENEW) then
--			tickcount = tickcount + HealPointsBS:GetBonus('DURATION_RENEW') / spell.class['interval']; -- 5 pc Oracle bonus + tier 5
		end

	elseif (className == "DRUID") then
		if (spell.class.classID == HealPointsSpells.SPELL_REJUV) then
			tickcount = tickcount + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NATSPL);
		elseif (spell.class.classID == HealPointsSpells.SPELL_REGR) then
			--tickcount = tickcount + HealPointsBS:GetBonus('DURATION_REGR') / spell.class['interval']; -- tier 5
			tickcount = tickcount + 2 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NATSPL);
		elseif (spell.class.classID == HealPointsSpells.SPELL_LIFEBL) then
			tickcount = tickcount + 2 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NATSPL);
		end
	end

	return tickcount;
end

--[[
	Returns the current player's Base Mana amount. 
	Base mana is the amount the player has before adding mana from intellect.
	@returns A number which is the player's base mana
	@remarks A players base mana is calculated by subtracting (Mana - ManaFromIntellect)
			Mana gained from intelliect is non-linear, with a knee at 20 Intellect.
--]]
function HealPointsSpells:GetBaseMana()
  local _, intellect, _, _ = UnitStat("player", 4); --4=Intellect
  return UnitManaMax("player") - (math.min(20, intellect) + 15 * (intellect - math.min(20, intellect))); 
end

function HealPointsSpells:getHighestSpellRank()
	--Legacy compatiblity function
	--Spells only have one rank now.
	--TODO: rewrite all logic referencing this function to stop using it.
	return 1;
end;

--[[
	Get the effect mana cost of a spell
	@param spell A HealPointsSpells:SPELLTABLE entry that indicates the spell to calculate the mana cost of
	@param spellCrit number The players current percentage chance at casting a critical spell
	@returns A number indicating the effective cost of a spell, after taking into account any mana cost reductions and mana refunds
	@example local mana = HealPointsSpells(hl, 4.03); --4.03=4.03% spell Crit Chance
	@remarks Each spell has a cost that is a percentage of "base Mana", leading to the displayed mana cost when hovering 
			over a spell. But some telents can reduce the mana cost of a spell, or refund some of the mana cost.
			This function returns the average effective mana cost of a spell, after taking these reductions into account.
]]--
function HealPointsSpells:getManaCost(spell, spellCrit)

	assert(spell);
	local spellManaCost = (spell['manaP'] / 100) * HealPointsSpells:GetBaseMana();
	
	--todo: look for any mana reduction effects, or mana refunding effects

	
	local percentCheaper = 0;
	local refund = 0;

	--[[
	spellCrit = spellCrit + self:getSpellCritBonus(spell);

	local _, className = UnitClass("player");

	if (className == "PALADIN") then
		percentCheaper = (spellCrit / 100) * 0.20 * HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.ILLUMI) * 0.6; -- Reduced mana cost for crits by 20% per rank
		if (spell.class.classID == HealPointsSpells.SPELL_HL) then
			refund = HealPointsBS:GetBonus('MANA_ABS_HL'); -- Straight mana reduction
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_HL') * 0.01; -- Tier 7
		elseif (spell.class.classID == HealPointsSpells.SPELL_HS) then
			percentCheaper = percentCheaper + HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.BENEDI) * 0.02; -- Instant spells 2% cheaper per rank
		end

	elseif (className == "PRIEST") then
		if (spell.class.classID == HealPointsSpells.SPELL_LH or spell.class.classID == HealPointsSpells.SPELL_HEAL or spell.class.classID == HealPointsSpells.SPELL_GH) then
			percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.IMPHEA) * 0.05; -- Reduced mana cost for slow heal by 5% per rank
		end
		if (spell.class.classID == HealPointsSpells.SPELL_FH or spell.class.classID == HealPointsSpells.SPELL_GH) then
			percentCheaper = percentCheaper + (spellCrit / 100) * (HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.HOLCON) * 0.10 + -- 10% chance per rank for free spell after crit
																														 HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.IMPHOC) * 0.05); -- 5% increased chance per rank
		end
		if (spell.class.classID == HealPointsSpells.SPELL_GH) then
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_GH') * 0.01; -- Tier 7
		end
		if (spell.class.classID == HealPointsSpells.SPELL_RENEW) then
			percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.MENAGI) * 0.02; -- Reduced mana cost for Renew by 2% per rank
		elseif (spell.class.classID == HealPointsSpells.SPELL_POH) then
			percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.HEAPRA) * 0.1; -- Reduced mana cost for Prayer of Healing by 10% per rank
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_POH') * 0.01; -- Tier 6
		elseif (spell.class.classID == HealPointsSpells.SPELL_HOLYNOVA or spell.class.classID == HealPointsSpells.SPELL_COH) then
			percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.MENAGI) * 0.02; -- Reduced mana cost for instants by 2% per rank		
		end

	elseif (className == "DRUID") then
		if (spell.class.classID == HealPointsSpells.SPELL_HT or spell.class.classID == HealPointsSpells.SPELL_REGROWTH or spell.class.classID == HealPointsSpells.SPELL_REJUV) then
				percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.MOONGL) * 0.03; -- Reduced mana cost for all spells by 3% per rank
		end
		if (spell.class.classID == HealPointsSpells.SPELL_HT) then
			percentCheaper = percentCheaper + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.TRASPI) * 0.02; -- Reduced mana cost for Healing Touch by 2% per rank
			if (HealPointsBS:GetBonus('MANA_REFUND_HT') > 0) then
				refund = refund + math.min(HealPointsBS:GetBonus('MANA_REFUND_HT'), (spell['manaP'] / 100) * HealPointSpells:getBaseMana() / (HealPointsBS:GetBonus('MANA_REFUND_HT') - 1)); -- Idol of Longevity
			end
		elseif (spell.class.classID == HealPointsSpells.SPELL_REGROWTH) then
			refund = refund + HealPointsBS:GetBonus('MANA_ABS_REGR'); -- Straight mana reduction
		elseif (spell.class.classID == HealPointsSpells.SPELL_REJUV) then
			refund = refund + HealPointsBS:GetBonus('MANA_ABS_REJUV'); -- Straight mana reduction
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_REJUV') * 0.01; -- Tier 7
		end		
		if (HealPointsUtil:isPlayerBuffUp(HealPointsLoc.BUFF_TREELI) and (spell.class.classID == HealPointsSpells.SPELL_REJUV or spell.class.classID == HealPointsSpells.SPELL_REGR or spell.class.classID == HealPointsSpells.SPELL_LIFEBL)) then
			percentCheaper = percentCheaper + 0.2; -- 20% cheaper in treeform
		end

	elseif (className == "SHAMAN") then
		percentCheaper = HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.TIDFOC) * 0.01; -- Reduced mana cost for all spells by 1% per rank
		if (spell.class.classID == HealPointsSpells.SPELL_HealingSurge) then
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_LHW') * 0.01; -- Tier 5
		elseif (spell.class.classID == HealPointsSpells.SPELL_HW) then
			refund = refund + HealPointsBS:GetBonus('MANA_ABS_HW'); -- Straight mana reduction
		elseif (spell.class.classID == HealPointsSpells.SPELL_CHAIN) then
			percentCheaper = percentCheaper + HealPointsBS:GetBonus('MANA_PC_CHAIN') * 0.01; -- Tier 6
			refund = refund + HealPointsBS:GetBonus('MANA_ABS_CHAIN'); -- Straight mana reduction
		end
		
	end
	--]]
		
	return spellManaCost * (1 - percentCheaper) - refund;
end


--[[
	Calculates the Bonus applied to a particular healing spell
	@returns number Returns a healing factor that applies to the passed spell (e.g. 1.07 implies 7% healing bonus)
--]]
function HealPointsSpells:getHealingPercentBonus(spell) 
	local percentBonus = StatLogic:GetStatMod("MOD_HEAL"); --e.g. 1

	--local _, className = UnitClass("player");

	--[[
	todo: look at talents

	if (className == "PALADIN") then
		percentBonus = HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.HEALIG) * 0.04; -- Increases avg for all three spells by 4% per rank
		if (spell.class.classID == HealPointsSpells.SPELL_FOL) then
			percentBonus = percentBonus + HealPointsBS:GetBonus('AVG_PC_FOL') * 0.01; -- Tier 6
		elseif (spell.class.classID == HealPointsSpells.SPELL_HS) then
			percentBonus = percentBonus + HealPointsBS:GetBonus('AVG_PC_HS') * 0.01; -- Arena		
		end
		percentBonus = 1 + percentBonus;

	elseif (className == "PRIEST") then
		percentBonus = 1 + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.SPIHEA) * 0.02;	-- Increased avg for all spells by 2% per rank
		percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.FOCUSP) * 0.02;	-- Increased avg for all spells by 2% per rank
		if (spell.class['type'] == 'hot' or (spell.class['type'] == 'aoe' and spell.class['time'] == 0)) then
				percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.TWINDI) * 0.01; -- Increased avg for instant spells by 1% per rank
		end
		if (spell.class.classID == HealPointsSpells.SPELL_RENEW) then
			percentBonus = percentBonus * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.IMPREN) * 0.05); -- Increased avg for Renew by 5% per rank
		elseif (spell.class.classID == HealPointsSpells.SPELL_GH) then
			percentBonus = percentBonus * (1 + HealPointsBS:GetBonus('AVG_PC_GH') * 0.01); -- Tier 6
		elseif (spell.class['type'] == 'aoe') then
			percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.DIVPRO) * 0.02; -- Increased avg for aoe spells by 2% per rank
		end
		-- Tested to be multiplicative for Renew

	elseif (className == "DRUID") then
		percentBonus = HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.GIOFNA) * 0.02; -- Increased avg for all spells by 2% per rank
		if (spell.class.classID == HealPointsSpells.SPELL_REJUV) then
			percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.IMPREJ) * 0.05; -- Increased avg for Rejuvenation by 5% per rank
			percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.GENESI) * 0.01; -- Increases hots by 1% per rank
		elseif (spell.class.classID == HealPointsSpells.SPELL_HT) then
			percentBonus = percentBonus + HealPointsBS:GetBonus('AVG_PC_HT') * 0.01; -- Tier 6
		end
		if (HealPointsUtil:isPlayerBuffUp(HealPointsLoc.BUFF_TREELI)) then
			percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.MASSHA) * 0.02; -- Increased avg by 2% per rank while in Tree Form
		end
		percentBonus = 1 + percentBonus; -- Tested not to be multiplicative for Rejuvenation

	elseif (className == "SHAMAN") then
		percentBonus = HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.PURIFI) * 0.02; -- Increased avg for all spells by 2% per rank
		if (spell.class.classID == HealPointsSpells.SPELL_CHAIN) then
			percentBonus = percentBonus + HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.IMPCHA) * 0.1; -- Increased avg for Chain heal by 10% per rank
			percentBonus = percentBonus + HealPointsBS:GetBonus('AVG_PC_CHAIN') * 0.01; -- Tier 6
		end
		if (spell.class.classID == HealPointsSpells.SPELL_CHAIN or spell.class.classID == HealPointsSpells.SPELL_HW) then
			percentBonus = percentBonus + HealPointsBS:GetBonus('AVG_PC_CHAIN_HW') * 0.01; -- Tier 7
		end
		percentBonus = 1 + percentBonus;
	end
	--]]
		
	return percentBonus;
end


--[[
	Returns
		healFactorHot:number
		healFactorBurst:number
--]]
function HealPointsSpells:getHealingFactor(spell)
	local levelFactor = 1 - ((20 - math.min(20, spell['level'])) * 0.0375);
	levelFactor = levelFactor * math.min(1, (spell['level'] + 11) / UnitLevel("player"));

	local timeFactor, timeFactor2;

	if (spell.class['coeff'] ~= nil) then
		timeFactor = spell.class['coeff'];
	elseif (spell.class['coeff1'] ~= nil) then
		timeFactor = spell.class['coeff1'];
		timeFactor2 = spell.class['coeff2'];
	else -- Fallback
		local castTime = spell.class['time'];
		if (castTime == nil) then
			castTime = spell['time'];
		end
		if (spell.class['type'] == 'regrowth') then
			timeFactor = 0.5 * castTime / 1.838;																				-- Experimental 3.0 (Burst)
			timeFactor2 = 0.5 * spell.class['tickcount'] * spell.class['interval'] / 8; -- Experimental 3.0 (HoT)
		elseif (spell.class['type'] == 'lifebloom') then
			timeFactor = 2 / 3;																												 -- Experimental 3.0 (HoT)
			timeFactor2 = 0.645;																												-- Experimental 3.0 (Burst)
		elseif (spell.class['type'] == 'hot') then
			timeFactor = spell.class['tickcount'] * spell.class['interval'] / 8;				-- Experimental 3.0
		elseif (spell.class['type'] == 'instant') then
			timeFactor = 1.5 / 1.854;																									 -- Experimental 3.0
		elseif (spell.class['type'] == 'aoe') then
			timeFactor = (math.max(1.5, castTime) / 1.87) / 2;													-- Experimental 3.0
			if (spell.class.classID == HealPointsSpells.SPELL_HOLYNOVA) then
				timeFactor = timeFactor * 3 / 4;																					-- Experimental 3.0
			end
		else -- normal / chain
			timeFactor = castTime / 1.47;																							 -- Experimental 3.0
		end
	end
	
	-- Empowered talents (strange difference between empowered healing and empowered rejuvenation)
	if (spell.class.classID == HealPointsSpells.SPELL_GH) then
		timeFactor = timeFactor + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.EMPHEA) * 0.08; -- +8% bonus healing per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_FH) then
		timeFactor = timeFactor + HealPointsTalents:getTalentRank(HealPointsTalents.PRIEST.EMPHEA) * 0.04; -- +4% bonus healing per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_HT) then
		timeFactor = timeFactor * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPOHT) * 0.1); -- +10% bonus healing per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_REJUV) then
		timeFactor = timeFactor * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPREJ) * 0.04); -- +4% bonus healing per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_REGR) then
		timeFactor = timeFactor * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPREJ) * 0.04); -- +4% bonus healing per rank (Burst)
		timeFactor2 = timeFactor2 * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPREJ) * 0.04); -- +4% bonus healing per rank (HoT)
	elseif (spell.class.classID == HealPointsSpells.SPELL_LIFEBL) then
		timeFactor = timeFactor * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPREJ) * 0.04); -- +4% bonus healing per rank 
		timeFactor2 = timeFactor2 * (1 + HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.EMPREJ) * 0.04); -- +4% bonus healing per rank 
	elseif (spell.class.classID == HealPointsSpells.SPELL_HW) then
		timeFactor = timeFactor + HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.TIDWAV) * 0.04; -- +4% bonus healing per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_HealingSurge) then
		timeFactor = timeFactor + HealPointsTalents:getTalentRank(HealPointsTalents.SHAMAN.TIDWAV) * 0.02; -- +2% bonus healing per rank	
	end
	
	if (timeFactor2 ~= nil) then
		return levelFactor * timeFactor, levelFactor * timeFactor2;
	else
		return levelFactor * timeFactor;
	end
end

--[[
	Returns the crit chance for a particular spell e.g. 30 = 30%
--]]
function HealPointsSpells:getSpellCritBonus(spell)
	local critBonus = 0;
	
	if (spell.class.classID == HealPointsSpells.SPELL_REGR) then
		critBonus = 10 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.IMPREG); -- Increases crit for Regrowth by 10% per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_HT) then
		critBonus = 2 * HealPointsTalents:getTalentRank(HealPointsTalents.DRUID.NATMAJ); -- Increases crit for Healing Touch by 2% per rank
	elseif (spell.class.classID == HealPointsSpells.SPELL_HL) then
		critBonus = 2 * HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.SANCLI); -- Increases crit for Holy Light and Holy Shock by 2% per rank
--		critBonus = critBonus + HealPointsBS:GetBonus('CRIT_HL'); -- Tier 6
	elseif (spell.class.classID == HealPointsSpells.SPELL_HS) then
		critBonus = 2 * HealPointsTalents:getTalentRank(HealPointsTalents.PALADIN.SANCLI); -- Increases crit for Holy Light and Holy Shock by 2% per rank	
		--critBonus = critBonus + HealPointsBS:GetBonus('CRIT_HS'); -- Tier 7
	elseif (spell.class.classID == HealPointsSpells.SPELL_FOL) then
		--critBonus = HealPointsBS:GetBonus('CRIT_FOL'); -- Arena set
	end
	
	return critBonus;
end

function HealPointsSpells:getHealingBonus(spell)
	local healBonus = 0;

	--[[
	--AVG_ABS_FOL "Increases spell power of Flash of Light by" doesn't exist in game anymore
	--if (spell.class.classID == HealPointsSpells.SPELL_FOL) then
--		healBonus = HealPointsBS:GetBonus('AVG_ABS_FOL'); -- Libram of Grace/Divinity
		
	--AVG_ABS_HL "Increases spell power of Holy Light by" doesn't exist in game anymore
	--elseif (spell.class.classID == HealPointsSpells.SPELL_HL) then
--		healBonus = HealPointsBS:GetBonus('AVG_ABS_HL');

	elseif (spell.class.classID == HealPointsSpells.SPELL_REJUV) then
		healBonus = HealPointsBS:GetBonus('AVG_ABS_REJUV'); -- Idol of Rejuvenation
	elseif (spell.class.classID == HealPointsSpells.SPELL_HT) then
		healBonus = HealPointsBS:GetBonus('AVG_ABS_HT'); -- Idol of Health
	elseif (spell.class.classID == HealPointsSpells.SPELL_HealingSurge) then
		healBonus = HealPointsBS:GetBonus('AVG_ABS_LHW'); -- Totem of Life/Sustaining
	elseif (spell.class.classID == HealPointsSpells.SPELL_HW) then
		healBonus = HealPointsBS:GetBonus('AVG_ABS_HW');
	elseif (spell.class.classID == HealPointsSpells.SPELL_CHAIN) then
		healBonus = HealPointsBS:GetBonus('AVG_ABS_CHAIN');
	end
	--]]
	
	return healBonus;
end

